@@ -5,7 +5,6 @@ sudo apt-get install -y \
     bison \
     cmake \
     gcc \
-    libcarp-assert-perl \
     libmodule-scandeps-perl \
     libparse-yapp-perl \
     libtool \
@@ -14,5 +13,6 @@ sudo apt-get install -y \
     perl \
     python-dev \
     python-setuptools \
+    python3-dev \
     python3-setuptools \
     swig
