@@ -1,8 +1,8 @@
 /*---------------------------------------------------------------------------*\
-**$Author: andrius $
-**$Date: 2019-12-27 14:57:31 +0200 (Fri, 27 Dec 2019) $
-**$Revision: 7625 $
-**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v2.12/src/components/codcif/cif2_lexer.c $
+**$Author: antanas $
+**$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $
+**$Revision: 8230 $
+**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/components/codcif/cif2_lexer.c $
 \*---------------------------------------------------------------------------*/
 
 /* exports: */
@@ -13,6 +13,7 @@
 #include <ctype.h>
 #include <cif_grammar_flex.h>
 #include <cif_grammar_y.h>
+#include <cif_lex_buffer.h>
 #include <cif2_grammar.tab.h>
 #include <common.h>
 #include <yy.h>
@@ -23,86 +24,36 @@
 
 static CIF_COMPILER *cif_cc;
 
-static char *current_line;
-static size_t currentl_line_length;
-static size_t current_pos;
-
-/* Inherited from the Flex scanner: */
-static char * thisTokenLine = NULL;
-static char * lastTokenLine = NULL;
-static char * currentLine = NULL;
-static int lineCnt = 1;
-static int currLine = 1;
-static int prevLine = 1;
-static int nextPos;
-
-static int lastTokenPos = 0;
-static int thisTokenPos = 0;
-
-static char *token = NULL;
-static size_t length = 0;
-
-static int ungot_ch = 0;
-
 /* was the last returned symbol a quoted string? */
 static int qstring_seen = 0;
 
-static int cif_mandated_line_length = 80;
-static int cif_mandated_tag_length = 74;
-static int report_long_items = 0;
+static size_t cif_mandated_tag_length = 74;
+static int report_long_tags = 0;
 
-int cif2_lexer_set_report_long_items( int flag )
+int cif2_lexer_set_report_long_tags( int flag )
 {
-    int old_value = report_long_items;
-    report_long_items = flag;
+    int old_value = report_long_tags;
+    report_long_tags = flag;
     return old_value;
 }
 
-int cif2_lexer_report_long_items( void )
+int cif2_lexer_report_long_tags( void )
 {
-    return report_long_items;
+    return report_long_tags;
 }
 
-int cif2_lexer_set_line_length_limit( int max_length )
+size_t cif2_lexer_set_tag_length_limit( size_t max_length )
 {
-    int old_value = cif_mandated_line_length;
-    cif_mandated_line_length = max_length;
-    return old_value;
-}
-
-int cif2_lexer_set_tag_length_limit( int max_length )
-{
-    int old_value = cif_mandated_tag_length;
+    size_t old_value = cif_mandated_tag_length;
     cif_mandated_tag_length = max_length;
     return old_value;
 }
 
-void cif2_flex_reset_counters( void )
-{
-    lineCnt = 1;
-    currLine = prevLine = 1;
-    current_pos = nextPos = 0;
-}
-/* end of old Flex scanner functions */
-
 void cif2_lexer_set_compiler( CIF_COMPILER *ccc )
 {
     cif_cc = ccc;
 }
 
-void cif2_lexer_cleanup( void )
-{
-    if( token ) freex( token );
-    token = NULL;
-    length = 0;
-}
-
-static void advance_mark( void )
-{
-    lastTokenPos = thisTokenPos;
-    thisTokenPos = current_pos - 1;
-}
-
 static int cif_lexer( FILE *in, cexception_t *ex );
 
 int cif2lex( void )
@@ -117,11 +68,6 @@ void cif2restart( void )
     /* FIXME: Nothing so far, to be expanded... */
 }
 
-static void pushchar( char **buf, size_t *length, size_t pos, int ch );
-static void ungetlinec( int ch, FILE *in );
-static int getlinec( FILE *in, cexception_t *ex );
-
-static char *clean_string( char *src, int is_textfield, cexception_t *ex );
 static void check_utf8( unsigned char *s );
 static char *check_and_clean( char *token, int is_textfield,
                               cexception_t *ex );
@@ -142,7 +88,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
         if( is_cif_space( ch ) || ch == '\0' ) {
             /* skip spaces: */
             prevchar = ch;
-            ch = getlinec( in, ex );
+            ch = getlinec( in, cif_cc, ex );
             if( is_cif_space( prevchar ) ) {
                 qstring_seen = 0;
             }
@@ -150,17 +96,17 @@ static int cif_lexer( FILE *in, cexception_t *ex )
         }
         switch( ch ) {
         case '\032': /* DOS EOF (^Z, Ctrl-Z) character */
-            thisTokenPos = current_pos > 0 ? current_pos - 1 : 0;
+            backstep_mark();
             if( cif_lexer_has_flags
                 (CIF_FLEX_LEXER_FIX_CTRL_Z) ) {
                 yywarning_token( cif_cc, "DOS EOF symbol ^Z was encountered and ignored",
-                                 cif2_flex_previous_line_number(), -1, ex );
+                                 cif_flex_previous_line_number(), -1, ex );
             } else {
                 cif2error( "DOS EOF symbol ^Z was encountered, "
                            "it is not permitted in CIFs" );
             }
             prevchar = ch;
-            ch = getlinec( in, ex );
+            ch = getlinec( in, cif_cc, ex );
             break;
         case '#':
             if( yy_flex_debug ) {
@@ -169,7 +115,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
             }
             /* skip comments: */
             while( ch != EOF && ch != '\n' && ch != '\r' ) {
-                ch = getlinec( in, ex );
+                ch = getlinec( in, cif_cc, ex );
                 if( yy_flex_debug ) {
                     putchar( ch );
                 }
@@ -177,7 +123,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
             if( ch == '\r' ) {
                 /* check and process the DOS newlines: */
                 int before = ch;
-                ch = getlinec( in, ex );
+                ch = getlinec( in, cif_cc, ex );
                 if( ch != '\n' ) {
                     ungetlinec( ch, in );
                     ch = before;
@@ -188,37 +134,38 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                 }
             }
             prevchar = ch;
-            ch = getlinec( in, ex );
+            ch = getlinec( in, cif_cc, ex );
             break;
         case '_':
             /* data name, or "tag": */
             advance_mark();
             pos = 0;
-            pushchar( &token, &length, pos++, ch );
-            pushchar( &token, &length, pos++,
-                      tolower(ch = getlinec( in, ex )) );
+            pushchar( pos++, ch );
+            pushchar( pos++,
+                      tolower(ch = getlinec( in, cif_cc, ex )) );
             while( !is_cif_space(ch) && ch != EOF ) {
-                pushchar( &token, &length, pos++,
-                          tolower(ch = getlinec( in, ex )) );
+                pushchar( pos++,
+                          tolower(ch = getlinec( in, cif_cc, ex )) );
             }
             ungetlinec( ch, in );
             pos --;
-            prevchar = token[pos-1];
-            pushchar( &token, &length, pos, '\0' );
-            check_utf8( (unsigned char *)token );
-            cif2lval.s = clean_string( token, /* is_textfield = */ 0, ex );
+            prevchar = cif_flex_token()[pos-1];
+            pushchar( pos, '\0' );
+            check_utf8( (unsigned char *)cif_flex_token() );
+            cif2lval.s = clean_string( cif_flex_token(), /* is_textfield = */ 0,
+                                       cif_cc, ex );
             /* Underscore must be followed by one or more non-empty
                symbol to pass as a correct tag name. */
             if( pos == 1 )
                 cif2error( "incorrect CIF syntax" );
             if( yy_flex_debug ) {
-                printf( ">>> TAG: '%s'\n", token );
+                printf( ">>> TAG: '%s'\n", cif_flex_token() );
             }
-            if( report_long_items ) {
+            if( report_long_tags ) {
                 if( strlen( cif2lval.s ) > cif_mandated_tag_length ) {
                     yynote_token( cif_cc, cxprintf( "data name '%s' exceeds %d characters",
                                       cif2lval.s, cif_mandated_tag_length ),
-                                  cif2_flex_previous_line_number(), -1, ex );
+                                  cif_flex_previous_line_number(), -1, ex );
                 }
             }
             qstring_seen = 0;
@@ -229,26 +176,27 @@ static int cif_lexer( FILE *in, cexception_t *ex )
         case '4': case '5': case '6': case '7': case '8': case '9':
             pos = 0;
             advance_mark();
-            pushchar( &token, &length, pos++, ch );
+            pushchar( pos++, ch );
             while( !is_cif_space( ch ) && ch != EOF &&
                     ch != '[' && ch != ']' && ch != '{' && ch != '}' ) {
-                pushchar( &token, &length, pos++, ch = getlinec( in, ex ));
+                pushchar( pos++, ch = getlinec( in, cif_cc, ex ));
             }
             ungetlinec( ch, in );
             pos --;
-            prevchar = token[pos-1];
-            pushchar( &token, &length, pos, '\0' );
-            check_utf8( (unsigned char*)token );
-            cif2lval.s = clean_string( token, /* is_textfield = */ 0, ex );
-            if( is_integer( token )) {
+            prevchar = cif_flex_token()[pos-1];
+            pushchar( pos, '\0' );
+            check_utf8( (unsigned char*)cif_flex_token() );
+            cif2lval.s = clean_string( cif_flex_token(), /* is_textfield = */ 0,\
+                                       cif_cc, ex );
+            if( is_integer( cif_flex_token() )) {
                 if( yy_flex_debug ) {
-                    printf( ">>> INTEGER: '%s'\n", token );
+                    printf( ">>> INTEGER: '%s'\n", cif_flex_token() );
                 }
                 qstring_seen = 0;
                 return _INTEGER_CONST;
-            } else if( is_real( token )) {
+            } else if( is_real( cif_flex_token() )) {
                 if( yy_flex_debug ) {
-                    printf( ">>> REAL: '%s'\n", token );
+                    printf( ">>> REAL: '%s'\n", cif_flex_token() );
                 }
                 qstring_seen = 0;
                 return _REAL_CONST;
@@ -260,7 +208,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                        real number without explicitly imposing that it 
                        should not contain any other symbols besides 
                        [.0-9] */
-                    printf( ">>> UQSTRING (not a number): '%s'\n", token );
+                    printf( ">>> UQSTRING (not a number): '%s'\n", cif_flex_token() );
                 }
                 qstring_seen = 0;
                 return _UQSTRING;
@@ -273,7 +221,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                 advance_mark();
                 pos = 0;
                 int quote_count = 1;
-                while( (ch = getlinec( in, ex )) == quote ) {
+                while( (ch = getlinec( in, cif_cc, ex )) == quote ) {
                     quote_count++;                    
                 }
                 ungetlinec( ch, in );
@@ -282,7 +230,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                     /* start of quote-delimited string */
                 } else if( quote_count == 2 ) {
                     /* empty quote-delimited string */
-                    ch = getlinec( in, ex );
+                    ch = getlinec( in, cif_cc, ex );
                     if( !is_cif_space( ch ) && ch != EOF && ch != ':' &&
                         ch != '[' && ch != ']' && ch != '{' && ch != '}' ) {
                         /* quoted string must be followed by a space
@@ -290,8 +238,8 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                         cif2error( "incorrect CIF syntax" );
                     }
                     ungetlinec( ch, in );
-                    token[0] = '\0';
-                    cif2lval.s = strdupx( token, ex );
+                    cif_flex_token()[0] = '\0';
+                    cif2lval.s = strdupx( cif_flex_token(), ex );
                     if( yy_flex_debug ) {
                         printf( ">>> *QSTRING (%c): ''\n",
                                 quote );
@@ -315,7 +263,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                     for( i = 0; i < quote_count - 6; i++ ) {
                         ungetlinec( quote, in );
                     }
-                    ch = getlinec( in, ex );
+                    ch = getlinec( in, cif_cc, ex );
                     if( !is_cif_space( ch ) && ch != EOF && ch != ':' &&
                         ch != '[' && ch != ']' && ch != '{' && ch != '}' ) {
                         /* quoted string must be followed by a space
@@ -323,8 +271,8 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                         cif2error( "incorrect CIF syntax" );
                     }
                     ungetlinec( ch, in );
-                    token[0] = '\0';
-                    cif2lval.s = strdupx( token, ex );
+                    cif_flex_token()[0] = '\0';
+                    cif2lval.s = strdupx( cif_flex_token(), ex );
                     type = quote == '"' ? _DQ3STRING : _SQ3STRING;
                     if( yy_flex_debug ) {
                         printf( ">>> *Q3STRING (%c): ''\n",
@@ -334,14 +282,14 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                     return type;
                 }
                 quote_count = 0;
-                while( (ch = getlinec( in, ex )) != EOF ) {
+                while( (ch = getlinec( in, cif_cc, ex )) != EOF ) {
                     if( type == _DQSTRING || type == _SQSTRING ) {
                         if( ch == '\n' || ch == '\r' ) {
                             break;
                         }
                         if( ch == quote ) {
                             /* properly terminated quote-delimited string: */
-                            ch = getlinec( in, ex );
+                            ch = getlinec( in, cif_cc, ex );
                             if( !is_cif_space( ch ) && ch != EOF && ch != ':' &&
                                 ch != '[' && ch != ']' && ch != '{' && ch != '}' ) {
                                 /* quoted string must be followed by a space
@@ -349,12 +297,12 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                                 cif2error( "incorrect CIF syntax" );
                             }
                             ungetlinec( ch, in );
-                            pushchar( &token, &length, pos, '\0' );
+                            pushchar( pos, '\0' );
                             cif2lval.s = check_and_clean
-                                ( token, /* is_textfield = */ 0, ex );
+                                ( cif_flex_token(), /* is_textfield = */ 0, ex );
                             if( yy_flex_debug ) {
                                 printf( ">>> *QSTRING (%c): '%s'\n",
-                                        quote, token );
+                                        quote, cif_flex_token() );
                             }
                             qstring_seen = 1;
                             return type;
@@ -371,12 +319,12 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                                  * or ':' in case of table keys */
                                 cif2error( "incorrect CIF syntax" );
                             }
-                            pushchar( &token, &length, pos-3, '\0' );
+                            pushchar( pos-3, '\0' );
                             cif2lval.s = check_and_clean
-                                ( token, /* is_textfield = */ 0, ex );
+                                ( cif_flex_token(), /* is_textfield = */ 0, ex );
                             if( yy_flex_debug ) {
                                 printf( ">>> *Q3STRING (%c): '%s'\n",
-                                        quote, token );
+                                        quote, cif_flex_token() );
                             }
                             qstring_seen = 1;
                             return type;
@@ -384,12 +332,12 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                             quote_count = 0;
                         }                        
                     }
-                    pushchar( &token, &length, pos++, ch );
+                    pushchar( pos++, ch );
                 }
                 /* Unterminated quoted string: */
-                prevchar = token[pos-1];
-                pushchar( &token, &length, pos, '\0' );
-                cif2lval.s = check_and_clean( token, /* is_textfield = */ 0,
+                prevchar = cif_flex_token()[pos-1];
+                pushchar( pos, '\0' );
+                cif2lval.s = check_and_clean( cif_flex_token(), /* is_textfield = */ 0,
                                             ex );
                 switch( quote ) {
                     case '"':
@@ -398,12 +346,12 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                             ) {
                             yywarning_token( cif_cc, "double-quoted string is missing "
                                              "a closing quote -- fixed",
-                                             cif2_flex_previous_line_number(), -1, ex );
+                                             cif_flex_previous_line_number(), -1, ex );
                         } else {
                             yyerror_token( cif_cc, "incorrect CIF syntax",
-                                           cif2_flex_current_line_number()-1,
-                                           cif2_flex_current_position()+1,
-                                           (char*)cif2_flex_previous_line(),
+                                           cif_flex_current_line_number()-1,
+                                           cif_flex_current_position()+1,
+                                           (char*)cif_flex_previous_line(),
                                            ex );
                         }
                         break;
@@ -413,12 +361,12 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                             ) {
                             yywarning_token( cif_cc, "single-quoted string is missing "
                                              "a closing quote -- fixed",
-                                             cif2_flex_previous_line_number(), -1, ex );
+                                             cif_flex_previous_line_number(), -1, ex );
                         } else {
                             yyerror_token( cif_cc, "incorrect CIF syntax",
-                                           cif2_flex_current_line_number()-1,
-                                           cif2_flex_current_position()+1,
-                                           (char*)cif2_flex_previous_line(),
+                                           cif_flex_current_line_number()-1,
+                                           cif_flex_current_position()+1,
+                                           (char*)cif_flex_previous_line(),
                                            ex );
                         }
                         break;
@@ -430,7 +378,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
         case '[': case ']': case '{': case '}':
             advance_mark();
             qstring_seen = 0;
-            int after = getlinec( in, ex );
+            int after = getlinec( in, cif_cc, ex );
             ungetlinec( after, in );
             if( (ch == ']' || ch == '}') &&
                 (after != EOF && !is_cif_space( after ) && after != ']' && after != '}') ) {
@@ -459,31 +407,31 @@ static int cif_lexer( FILE *in, cexception_t *ex )
             if( ch == ';' && ( prevchar == '\n' || prevchar == '\0' ) ) {
                 /* multi-line text field: */
                 advance_mark();
-                ssize_t textfield_start = cif2_flex_current_line_number();
+                ssize_t textfield_start = cif_flex_current_line_number();
                 pos = 0;
                 while( ch != EOF ) {
                     prevchar = ch;
-                    ch = getlinec( in, ex );
+                    ch = getlinec( in, cif_cc, ex );
                     if( ch == ';' &&
                         ( prevchar == '\n' || prevchar == '\r' )) {
                         /* end of the text field detected: */
                         prevchar = ch;
-                        int after = getlinec( in, ex );
+                        int after = getlinec( in, cif_cc, ex );
                         ungetlinec( after, in );
                         if( !is_cif_space( after ) && after != EOF ) {
                             cif2error( "incorrect CIF syntax" );
                         }
-                        token[pos-1] = '\0'; /* delete the last '\n' char */
+                        cif_flex_token()[pos-1] = '\0'; /* delete the last '\n' char */
                         if( yy_flex_debug ) {
-                            printf( ">>> TEXT FIELD: '%s'\n", token );
+                            printf( ">>> TEXT FIELD: '%s'\n", cif_flex_token() );
                         }
-                        check_utf8( (unsigned char*)token );
-                        cif2lval.s = clean_string( token, /* is_textfield = */ 1,
-                                                 ex );
+                        check_utf8( (unsigned char*)cif_flex_token() );
+                        cif2lval.s = clean_string( cif_flex_token(), /* is_textfield = */ 1,
+                                                   cif_cc, ex );
                         qstring_seen = 0;
                         return _TEXT_FIELD;
                     }
-                    pushchar( &token, &length, pos++, ch );
+                    pushchar( pos++, ch );
                 }
                 /* Unterminated text field: */
                 yyerror_token( cif_cc,
@@ -491,7 +439,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                                "text field starting in line %d, "
                                "possible runaway closing semicolon (';')",
                                textfield_start ),
-                               cif2_flex_current_line_number()-1, -1,
+                               cif_flex_current_line_number()-1, -1,
                                NULL, ex );
             }
             /* else this is an ordinary unquoted string -- drop
@@ -500,46 +448,46 @@ static int cif_lexer( FILE *in, cexception_t *ex )
         default:
             pos = 0;
             advance_mark();
-            pushchar( &token, &length, pos++, ch );
+            pushchar( pos++, ch );
             int is_container_code = 0;
             while( !is_cif_space( ch ) && ch != EOF &&
                    (is_container_code ||
                     (ch != '[' && ch != ']' && ch != '{' && ch != '}')) ) {
-                pushchar( &token, &length, pos++, ch = getlinec( in, ex ));
+                pushchar( pos++, ch = getlinec( in, cif_cc, ex ));
                 if( pos == 5 &&
-                    ( starts_with_keyword( "data_", token ) ||
-                      starts_with_keyword( "save_", token ) ) ) {
+                    ( starts_with_keyword( "data_", cif_flex_token() ) ||
+                      starts_with_keyword( "save_", cif_flex_token() ) ) ) {
                     is_container_code = 1;
                 }
             }
             ungetlinec( ch, in );
-            prevchar = token[pos-1];
+            prevchar = cif_flex_token()[pos-1];
             pos --;
-            assert( pos < length );
+            // assert( pos < length );
             assert( pos >= 0 );
-            token[pos] = '\0';
-            if( starts_with_keyword( "data_", token ) ) {
+            cif_flex_token()[pos] = '\0';
+            if( starts_with_keyword( "data_", cif_flex_token() ) ) {
                 /* data block header: */
-                if( strlen( token ) == 5 ) {
+                if( pos == 5 ) {
                     if( cif_lexer_has_flags(CIF_FLEX_LEXER_FIX_DATABLOCK_NAMES) ) {
                         yywarning_token( cif_cc, "zero-length data block name detected "
                                          "-- ignored",
-                                         cif2_flex_previous_line_number(), -1, ex );
+                                         cif_flex_previous_line_number(), -1, ex );
                     } else {
                         cif2error( "zero-length data block name detected" );
                     }
                 }
                 if( yy_flex_debug ) {
-                    printf( ">>> DATA_: '%s'\n", token + 5 );
+                    printf( ">>> DATA_: '%s'\n", cif_flex_token() + 5 );
                 }
-                check_utf8( (unsigned char*)token );
-                cif2lval.s = clean_string( token + 5, /* is_textfield = */ 0,
-                                         ex );
+                check_utf8( (unsigned char*)cif_flex_token() );
+                cif2lval.s = clean_string( cif_flex_token() + 5, /* is_textfield = */ 0,
+                                           cif_cc, ex );
                 qstring_seen = 0;
                 return _DATA_;
-            } else if( starts_with_keyword( "save_", token )) {
+            } else if( starts_with_keyword( "save_", cif_flex_token() )) {
                 /* save frame header or terminator: */
-                if( strlen( token ) == 5 /* strlen( "save_" ) */ ) {
+                if( pos == 5 /* strlen( "save_" ) */ ) {
                     /* This is a save frame terminator: */
                     if( yy_flex_debug ) {
                         printf( ">>> SAVE_\n" );
@@ -549,50 +497,48 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                     return _SAVE_FOOT;
                 } else {
                     if( yy_flex_debug ) {
-                        printf( ">>> SAVE_: '%s'\n", token + 5 );
+                        printf( ">>> SAVE_: '%s'\n", cif_flex_token() + 5 );
                     }
-                    check_utf8( (unsigned char*)token );
-                    cif2lval.s = clean_string( token + 5, /* is_textfield = */ 0,
-                                             ex );
+                    check_utf8( (unsigned char*)cif_flex_token() );
+                    cif2lval.s = clean_string( cif_flex_token() + 5, /* is_textfield = */ 0,
+                                               cif_cc, ex );
                     qstring_seen = 0;
                     return _SAVE_HEAD;
                 }
-            } else if( starts_with_keyword( "loop_", token ) &&
-                strlen( token ) == 5) {
+            } else if( starts_with_keyword( "loop_", cif_flex_token() ) && pos == 5) {
                 /* loop header: */
                 if( yy_flex_debug ) {
                     printf( ">>> LOOP_\n" );
                 }
-                check_utf8( (unsigned char*)token );
-                cif2lval.s = clean_string( token, /* is_textfield = */ 0, ex );
+                check_utf8( (unsigned char*)cif_flex_token() );
+                cif2lval.s = clean_string( cif_flex_token(), /* is_textfield = */ 0,
+                                           cif_cc, ex );
                 qstring_seen = 0;
                 return _LOOP_;
-            } else if( starts_with_keyword( "stop_", token ) &&
-                strlen( token ) == 5 ) {
+            } else if( starts_with_keyword( "stop_", cif_flex_token() ) && pos == 5 ) {
                 /* stop field: */
                 cif2error( "STOP_ symbol detected -- "
                          "it is not acceptable in CIF v2.0" );
-            } else if( starts_with_keyword( "global_", token ) &&
-                strlen( token ) == 7 ) {
+            } else if( starts_with_keyword( "global_", cif_flex_token() ) && pos == 7 ) {
                 /* global field: */
                 cif2error( "GLOBAL_ symbol detected -- "
                          "it is not acceptable in CIF v2.0" );
             } else {
-                cif2lval.s = check_and_clean( token, /* is_textfield = */ 0,
+                cif2lval.s = check_and_clean( cif_flex_token(), /* is_textfield = */ 0,
                                             ex );
                 qstring_seen = 0;
-                if( token[0] == '$' ) {
+                if( cif_flex_token()[0] == '$' ) {
                     /* dollar is a reserved symbol, unquoted strings
                        may not start with it: */
                     cif2error( "dollar symbol ('$') must not start an "
                                "unquoted string" );
                     if( yy_flex_debug ) {
-                        printf( ">>> SQSTRING (corrected dollar): '%s'\n", token );
+                        printf( ">>> SQSTRING (corrected dollar): '%s'\n", cif_flex_token() );
                     }
                     return _SQSTRING;
                 } else {
                     if( yy_flex_debug ) {
-                        printf( ">>> UQSTRING: '%s'\n", token );
+                        printf( ">>> UQSTRING: '%s'\n", cif_flex_token() );
                     }
                     return _UQSTRING;
                 }
@@ -604,176 +550,6 @@ static int cif_lexer( FILE *in, cexception_t *ex )
     return 0;
 }
 
-static void pushchar( char **buf, size_t *length, size_t pos, int ch )
-{
-    char *str;
-
-    if( !buf || pos >= *length ) {
-        size_t max_size = (size_t)-1;
-        if( *length == 0 ) {
-            *length = 128;
-        } else {
-            if( *length > max_size / 2 ) {
-                cexception_raise( NULL, -99, "cannot double the buffer size" );
-            }
-        }
-        *length *= 2;
-        if( yy_flex_debug ) {
-            printf( ">>> reallocating lex token buffer to %lu\n", *length );
-        }
-        *buf = reallocx( *buf, *length, NULL );
-    }
-
-    str = *buf;
-
-    assert( pos < *length );
-    str[pos] = ch;
-}
-
-void ungetlinec( int ch, FILE *in )
-{
-    ungot_ch = 1;
-    /* CHECKME: see if the lines are switched correctly when '\n' is
-       pushed back at the end of a DOS new line: */
-    if( ch == '\n' || ch == '\r' ) {
-        thisTokenLine = lastTokenLine;
-        currLine --;
-    }
-    ungetc( ch, in );
-}
-
-static int getlinec( FILE *in, cexception_t *ex )
-{
-    int ch = getc( in );
-    static char prevchar;
-
-    if( ch != EOF && !ungot_ch ) {
-        if( ch == '\n' || ch == '\r' ) {
-            if( ch == '\r' || (ch == '\n' && prevchar != '\r' &&
-                               prevchar != '\n')) {
-                prevLine = lineCnt;
-                if( lastTokenLine )
-                    freex( lastTokenLine );
-                if( current_line ) {
-                    lastTokenLine = strdupx( current_line, ex );
-                    if( report_long_items ) {
-                        if( strlen( current_line ) > cif_mandated_line_length ) {
-                            yynote_token( cif_cc, cxprintf( "line exceeds %d characters", 
-                                              cif_mandated_line_length ),
-                                  cif2_flex_previous_line_number(), -1, ex );
-                        }
-                    }
-                } else {
-                    lastTokenLine = NULL;
-                }
-            }
-            if( ch == '\r' || (ch == '\n' && prevchar != '\r' )) {
-                lineCnt ++;
-                current_pos = 0;
-            }
-            pushchar( &current_line, &currentl_line_length, 0, '\0' );
-        } else {
-            pushchar( &current_line, &currentl_line_length, current_pos++, ch );
-            pushchar( &current_line, &currentl_line_length, current_pos, '\0' );
-        }
-        prevchar = ch;
-        currentLine = thisTokenLine = current_line;
-        /* printf( ">>> lastTokenLine = '%s'\n", lastTokenLine ); */
-        /* printf( ">>> thisTokenLine = '%s'\n", thisTokenLine ); */
-    }
-    currLine = lineCnt;
-    ungot_ch = 0;
-    return ch;
-}
-
-int cif2_flex_current_line_number( void ) { return currLine; }
-int cif2_flex_previous_line_number( void ) { return prevLine; }
-void cif2_flex_set_current_line_number( ssize_t line ) { lineCnt = line; }
-int cif2_flex_current_position( void ) { return thisTokenPos; }
-int cif2_flex_previous_position( void ) { return lastTokenPos; }
-void cif2_flex_set_current_position( ssize_t pos ) { current_pos = pos - 1; }
-const char *cif2_flex_current_line( void ) { return thisTokenLine; }
-const char *cif2_flex_previous_line( void ) { return lastTokenLine; }
-
-static char *clean_string( char *src, int is_textfield, cexception_t *ex )
-{
-    int DELTA = 8;
-    ssize_t length = strlen( src );
-    char *volatile new = mallocx( length + 1, ex );
-    char *dest = new;
-    char *start = src;
-    int non_ascii_explained = 0;
-
-    cexception_t inner;
-    cexception_guard( inner ) {
-        while( *src != '\0' ) {
-            if( ( (*src & 255 ) < 32 || (*src & 255 ) == 127 ||
-                ( !cif_lexer_has_flags(CIF_FLEX_LEXER_ALLOW_HIGH_CHARS) &&
-                  (*src & 255 ) > 127 ) )
-                && (*src & 255 ) != '\n'
-                && (*src & 255 ) != '\t'
-                && (*src & 255 ) != '\r' ) {
-                if( cif_lexer_has_flags
-                (CIF_FLEX_LEXER_FIX_NON_ASCII_SYMBOLS)) {
-                    /* Do magic with non-ASCII symbols */
-                    *dest = '\0';
-                    length += DELTA;
-                    new = reallocx( new, length + 1, &inner );
-                    strcat( new, cxprintf( "&#x%04X;", *src & 255 ) );
-                    dest = new + strlen( new ) - 1;
-                    if( non_ascii_explained == 0 ) {
-                        if( is_textfield == 0 ) {
-                            print_message( cif_cc, "WARNING", "non-ASCII symbols "
-                                           "encountered in the text", ":",
-                                           cif2_flex_current_line_number(),
-                                           cif2_flex_current_position()+1,
-                                           ex );
-                            print_trace( cif_cc, (char*)cif2_flex_current_line(),
-                                         cif2_flex_current_position()+1, ex );
-                        } else {
-                            print_message( cif_cc, "WARNING", "non-ASCII symbols "
-                                           "encountered in the text field -- "
-                                           "replaced with XML entities", ":",
-                                           cif2_flex_current_line_number(),
-                                           -1, ex );
-                            print_current_text_field( cif_cc, start, ex );
-                        }
-                        non_ascii_explained = 1;
-                    }
-                } else {
-                    if( non_ascii_explained == 0 ) {
-                        if( is_textfield == 0 ) {
-                            cif2error( "incorrect CIF syntax" );
-                        } else {
-                            print_message( cif_cc, "ERROR", "non-ASCII symbols "
-                                           "encountered "
-                                           "in the text field", ":",
-                                           cif2_flex_current_line_number(),
-                                           -1, ex );
-                            print_current_text_field( cif_cc, start, ex );
-                            cif_compiler_increase_nerrors( cif_cc );
-                        }
-                        non_ascii_explained = 1;
-                    }
-                    dest--; /* Omit non-ASCII symbols */
-                }
-            } else if( (*src & 255) == '\r' ) {
-                dest--; /* Skip carriage return symbols */
-            } else {
-                *dest = *src;
-            }
-            src++;
-            dest++;
-        }
-    }
-    cexception_catch {
-        freex( new );
-        cexception_reraise( inner, ex );
-    }
-    *dest = '\0';
-    return new;
-}
-
 static int string_has_control_chars( unsigned char *s )
 {
     if( !s ) return 0;
@@ -844,7 +620,7 @@ static char *check_and_clean( char *token, int is_textfield, cexception_t *ex )
         ( (unsigned char*)token )) {
         if( cif_lexer_has_flags
             (CIF_FLEX_LEXER_FIX_NON_ASCII_SYMBOLS) ) {
-            s = clean_string( token, is_textfield, ex );
+            s = clean_string( token, is_textfield, cif_cc, ex );
         } else {
             cif2error( "incorrect CIF syntax" );
             s = strdupx( token, ex );
