@@ -1,8 +1,8 @@
 /*---------------------------------------------------------------------------*\
-**$Author: andrius $
-**$Date: 2019-12-27 14:57:31 +0200 (Fri, 27 Dec 2019) $
-**$Revision: 7625 $
-**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v2.12/src/components/codcif/cif_lexer.c $
+**$Author: antanas $
+**$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $
+**$Revision: 8230 $
+**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/components/codcif/cif_lexer.c $
 \*---------------------------------------------------------------------------*/
 
 /* exports: */
@@ -14,6 +14,7 @@
 #include <cif_grammar_flex.h>
 #include <cif_grammar_y.h>
 #include <cif_grammar.tab.h>
+#include <cif_lex_buffer.h>
 #include <common.h>
 #include <yy.h>
 #include <cxprintf.h>
@@ -23,83 +24,33 @@
 
 static CIF_COMPILER *cif_cc;
 
-static char *current_line;
-static size_t currentl_line_length;
-static size_t current_pos;
+static size_t cif_mandated_tag_length = 74;
+static int report_long_tags = 0;
 
-/* Inherited from the Flex scanner: */
-static char * thisTokenLine = NULL;
-static char * lastTokenLine = NULL;
-static char * currentLine = NULL;
-static int lineCnt = 1;
-static int currLine = 1;
-static int prevLine = 1;
-static int nextPos;
-
-static int lastTokenPos = 0;
-static int thisTokenPos = 0;
-
-static char *token = NULL;
-static size_t length = 0;
-
-static int ungot_ch = 0;
-
-static int cif_mandated_line_length = 80;
-static int cif_mandated_tag_length = 74;
-static int report_long_items = 0;
-
-int cif_lexer_set_report_long_items( int flag )
+int cif_lexer_set_report_long_tags( int flag )
 {
-    int old_value = report_long_items;
-    report_long_items = flag;
+    int old_value = report_long_tags;
+    report_long_tags = flag;
     return old_value;
 }
 
-int cif_lexer_report_long_items( void )
+int cif_lexer_report_long_tags( void )
 {
-    return report_long_items;
+    return report_long_tags;
 }
 
-int cif_lexer_set_line_length_limit( int max_length )
+size_t cif_lexer_set_tag_length_limit( size_t max_length )
 {
-    int old_value = cif_mandated_line_length;
-    cif_mandated_line_length = max_length;
-    return old_value;
-}
-
-int cif_lexer_set_tag_length_limit( int max_length )
-{
-    int old_value = cif_mandated_tag_length;
+    size_t old_value = cif_mandated_tag_length;
     cif_mandated_tag_length = max_length;
     return old_value;
 }
 
-void cif_flex_reset_counters( void )
-{
-    lineCnt = 1;
-    currLine = prevLine = 1;
-    current_pos = nextPos = 0;
-}
-/* end of old Flex scanner functions */
-
 void cif_lexer_set_compiler( CIF_COMPILER *ccc )
 {
     cif_cc = ccc;
 }
 
-void cif_lexer_cleanup( void )
-{
-    if( token ) freex( token );
-    token = NULL;
-    length = 0;
-}
-
-static void advance_mark( void )
-{
-    lastTokenPos = thisTokenPos;
-    thisTokenPos = current_pos - 1;
-}
-
 static int cif_lexer( FILE *in, cexception_t *ex );
 
 int ciflex( void )
@@ -114,12 +65,6 @@ void cifrestart( void )
     /* FIXME: Nothing so far, to be expanded... */
 }
 
-static void pushchar( char **buf, size_t *length, size_t pos, int ch );
-static void ungetlinec( int ch, FILE *in );
-static int getlinec( FILE *in, cexception_t *ex );
-
-static char *clean_string( char *src, int is_textfield, cexception_t *ex );
-
 static int cif_lexer( FILE *in, cexception_t *ex )
 {
     int ch = '\0';
@@ -136,12 +81,12 @@ static int cif_lexer( FILE *in, cexception_t *ex )
         if( is_cif_space( ch ) || ch == '\0' ) {
             /* skip spaces: */
             prevchar = ch;
-            ch = getlinec( in, ex );
+            ch = getlinec( in, cif_cc, ex );
             continue;
         }
         switch( ch ) {
         case '\032': /* DOS EOF (^Z, Ctrl-Z) character */
-            thisTokenPos = current_pos > 0 ? current_pos - 1 : 0;
+            backstep_mark();
             if( cif_lexer_has_flags
                 (CIF_FLEX_LEXER_FIX_CTRL_Z) ) {
                 yywarning_token( cif_cc, "DOS EOF symbol ^Z was encountered and ignored",
@@ -151,7 +96,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                           "it is not permitted in CIFs" );
             }
             prevchar = ch;
-            ch = getlinec( in, ex );
+            ch = getlinec( in, cif_cc, ex );
             break;
         case '#':
             if( yy_flex_debug ) {
@@ -159,9 +104,9 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                 putchar( ch );
             }
             /* skip comments: */
-            pos = current_pos;
+            pos = cif_flex_current_mark_position();
             while( ch != EOF && ch != '\n' && ch != '\r' ) {
-                ch = getlinec( in, ex );
+                ch = getlinec( in, cif_cc, ex );
                 pos ++;
                 if( yy_flex_debug ) {
                     putchar( ch );
@@ -181,7 +126,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
             if( ch == '\r' ) {
                 /* check and process the DOS newlines: */
                 int before = ch;
-                ch = getlinec( in, ex );
+                ch = getlinec( in, cif_cc, ex );
                 if( ch != '\n' ) {
                     ungetlinec( ch, in );
                     ch = before;
@@ -192,30 +137,30 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                 }
             }
             prevchar = ch;
-            ch = getlinec( in, ex );
+            ch = getlinec( in, cif_cc, ex );
             break;
         case '_':
             /* data name, or "tag": */
             advance_mark();
             pos = 0;
-            pushchar( &token, &length, pos++, ch );
+            pushchar( pos++, ch );
             while( !is_cif_space(ch) && ch != EOF ) {
-                ch = getlinec( in, ex );
-                pushchar( &token, &length, pos++, tolower(ch) );
+                ch = getlinec( in, cif_cc, ex );
+                pushchar( pos++, tolower(ch) );
             }
             ungetlinec( ch, in );
             pos --;
-            prevchar = token[pos-1];
-            pushchar( &token, &length, pos, '\0' );
-            ciflval.s = clean_string( token, /* is_textfield = */ 0, ex );
+            prevchar = cif_flex_token()[pos-1];
+            pushchar( pos, '\0' );
+            ciflval.s = clean_string( cif_flex_token(), /* is_textfield = */ 0, cif_cc, ex );
             /* Underscore must be followed by one or more non-empty
                symbol to pass as a correct tag name. */
             if( pos == 1 )
                 ciferror( "incorrect CIF syntax" );
             if( yy_flex_debug ) {
-                printf( ">>> TAG: '%s'\n", token );
+                printf( ">>> TAG: '%s'\n", cif_flex_token() );
             }
-            if( report_long_items ) {
+            if( report_long_tags ) {
                 if( strlen( ciflval.s ) > cif_mandated_tag_length ) {
                     yynote_token( cif_cc, cxprintf( "data name '%s' exceeds %d characters",
                                       ciflval.s, cif_mandated_tag_length ),
@@ -229,23 +174,23 @@ static int cif_lexer( FILE *in, cexception_t *ex )
         case '4': case '5': case '6': case '7': case '8': case '9':
             pos = 0;
             advance_mark();
-            pushchar( &token, &length, pos++, ch );
+            pushchar( pos++, ch );
             while( !is_cif_space( ch ) && ch != EOF ) {
-                pushchar( &token, &length, pos++, ch = getlinec( in, ex ));
+                pushchar( pos++, ch = getlinec( in, cif_cc, ex ));
             }
             ungetlinec( ch, in );
             pos --;
-            prevchar = token[pos-1];
-            pushchar( &token, &length, pos, '\0' );
-            ciflval.s = clean_string( token, /* is_textfield = */ 0, ex );
-            if( is_integer( token )) {
+            prevchar = cif_flex_token()[pos-1];
+            pushchar( pos, '\0' );
+            ciflval.s = clean_string( cif_flex_token(), /* is_textfield = */ 0, cif_cc, ex );
+            if( is_integer( cif_flex_token() )) {
                 if( yy_flex_debug ) {
-                    printf( ">>> INTEGER: '%s'\n", token );
+                    printf( ">>> INTEGER: '%s'\n", cif_flex_token() );
                 }
                 return _INTEGER_CONST;
-            } else if( is_real( token )) {
+            } else if( is_real( cif_flex_token() )) {
                 if( yy_flex_debug ) {
-                    printf( ">>> REAL: '%s'\n", token );
+                    printf( ">>> REAL: '%s'\n", cif_flex_token() );
                 }
                 return _REAL_CONST;
             } else {
@@ -256,7 +201,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                        real number without explicitly imposing that it 
                        should not contain any other symbols besides 
                        [.0-9] */
-                    printf( ">>> UQSTRING (not a number): '%s'\n", token );
+                    printf( ">>> UQSTRING (not a number): '%s'\n", cif_flex_token() );
                 }
                 return _UQSTRING;
             }
@@ -267,24 +212,24 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                 int quote = ch;
                 advance_mark();
                 pos = 0;
-                while( (ch = getlinec( in, ex )) != EOF ) {
+                while( (ch = getlinec( in, cif_cc, ex )) != EOF ) {
                     if( ch == '\n' || ch == '\r' )
                         break;
                     if( ch != quote ) {
-                        pushchar( &token, &length, pos++, ch );
+                        pushchar( pos++, ch );
                     } else {
                         /* check if the quote terminates the string: */
                         int before = ch;
-                        ch = getlinec( in, ex );
+                        ch = getlinec( in, cif_cc, ex );
                         if( ch == EOF || is_cif_space(ch) ) {
                             /* The quoted string is properly terminated: */
                             ungetlinec( ch, in );
-                            pushchar( &token, &length, pos, '\0' );
+                            pushchar( pos, '\0' );
                             ciflval.s = clean_string
-                                ( token, /* is_textfield = */ 0, ex );
+                                ( cif_flex_token(), /* is_textfield = */ 0, cif_cc, ex );
                             if( yy_flex_debug ) {
                                 printf( ">>> *QSTRING (%c): '%s'\n",
-                                        quote, token );
+                                        quote, cif_flex_token() );
                             }
                             return quote == '"' ? _DQSTRING : _SQSTRING;
                         } else {
@@ -292,14 +237,15 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                                string, it is a part of the value: */
                             ungetlinec( ch, in );
                             prevchar = before;
-                            pushchar( &token, &length, pos++, before );
+                            pushchar( pos++, before );
                         }
                     }
                 }
                 /* Unterminated quoted string: */
-                prevchar = token[pos-1];
-                pushchar( &token, &length, pos, '\0' );
-                ciflval.s = clean_string( token, /* is_textfield = */ 0, ex );
+                prevchar = cif_flex_token()[pos-1];
+                pushchar( pos, '\0' );
+                ciflval.s = clean_string( cif_flex_token(), /* is_textfield = */ 0,
+                                          cif_cc, ex );
                 switch( quote ) {
                     case '"':
                         if( cif_lexer_has_flags
@@ -343,25 +289,25 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                 pos = 0;
                 while( ch != EOF ) {
                     prevchar = ch;
-                    ch = getlinec( in, ex );
+                    ch = getlinec( in, cif_cc, ex );
                     if( ch == ';' &&
                         ( prevchar == '\n' || prevchar == '\r' )) {
                         /* end of the text field detected: */
                         prevchar = ch;
-                        int after = getlinec( in, ex );
+                        int after = getlinec( in, cif_cc, ex );
                         ungetlinec( after, in );
                         if( !is_cif_space( after ) && after != EOF ) {
                             ciferror( "incorrect CIF syntax" );
                         }
-                        token[pos-1] = '\0'; /* delete the last '\n' char */
+                        cif_flex_token()[pos-1] = '\0'; /* delete the last '\n' char */
                         if( yy_flex_debug ) {
-                            printf( ">>> TEXT FIELD: '%s'\n", token );
+                            printf( ">>> TEXT FIELD: '%s'\n", cif_flex_token() );
                         }
-                        ciflval.s = clean_string( token, /* is_textfield = */ 1,
-                                                  ex );
+                        ciflval.s = clean_string( cif_flex_token(), /* is_textfield = */ 1,
+                                                  cif_cc, ex );
                         return _TEXT_FIELD;
                     }
-                    pushchar( &token, &length, pos++, ch );
+                    pushchar( pos++, ch );
                 }
                 /* Unterminated text field: */
                 yyerror_token( cif_cc,
@@ -378,19 +324,19 @@ static int cif_lexer( FILE *in, cexception_t *ex )
         default:
             pos = 0;
             advance_mark();
-            pushchar( &token, &length, pos++, ch );
+            pushchar( pos++, ch );
             while( !is_cif_space( ch ) && ch != EOF ) {
-                pushchar( &token, &length, pos++, ch = getlinec( in, ex ));
+                pushchar( pos++, ch = getlinec( in, cif_cc, ex ));
             }
             ungetlinec( ch, in );
-            prevchar = token[pos-1];
+            prevchar = cif_flex_token()[pos-1];
             pos --;
-            assert( pos < length );
+            // assert( pos < length );
             assert( pos >= 0 );
-            token[pos] = '\0';
-            if( starts_with_keyword( "data_", token ) ) {
+            cif_flex_token()[pos] = '\0';
+            if( starts_with_keyword( "data_", cif_flex_token() ) ) {
                 /* data block header: */
-                if( strlen( token ) == 5 ) {
+                if( pos == 5 ) {
                     if( cif_lexer_has_flags(CIF_FLEX_LEXER_FIX_DATABLOCK_NAMES) ) {
                         yywarning_token( cif_cc, "zero-length data block name detected "
                                          "-- ignored",
@@ -400,14 +346,14 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                     }
                 }
                 if( yy_flex_debug ) {
-                    printf( ">>> DATA_: '%s'\n", token + 5 );
+                    printf( ">>> DATA_: '%s'\n", cif_flex_token() + 5 );
                 }
-                ciflval.s = clean_string( token + 5, /* is_textfield = */ 0,
-                                          ex );
+                ciflval.s = clean_string( cif_flex_token() + 5, /* is_textfield = */ 0,
+                                          cif_cc, ex );
                 return _DATA_;
-            } else if( starts_with_keyword( "save_", token )) {
+            } else if( starts_with_keyword( "save_", cif_flex_token() )) {
                 /* save frame header or terminator: */
-                if( strlen( token ) == 5 /* strlen( "save_" ) */ ) {
+                if( pos == 5 /* strlen( "save_" ) */ ) {
                     /* This is a save frame terminator: */
                     if( yy_flex_debug ) {
                         printf( ">>> SAVE_\n" );
@@ -416,32 +362,29 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                     return _SAVE_FOOT;
                 } else {
                     if( yy_flex_debug ) {
-                        printf( ">>> SAVE_: '%s'\n", token + 5 );
+                        printf( ">>> SAVE_: '%s'\n", cif_flex_token() + 5 );
                     }
-                    ciflval.s = clean_string( token + 5, /* is_textfield = */ 0,
-                                              ex );
+                    ciflval.s = clean_string( cif_flex_token() + 5, /* is_textfield = */ 0,
+                                              cif_cc, ex );
                     return _SAVE_HEAD;
                 }
-            } else if( starts_with_keyword( "loop_", token ) &&
-                strlen( token ) == 5) {
+            } else if( starts_with_keyword( "loop_", cif_flex_token() ) && pos == 5) {
                 /* loop header: */
                 if( yy_flex_debug ) {
                     printf( ">>> LOOP_\n" );
                 }
-                ciflval.s = clean_string( token, /* is_textfield = */ 0, ex );
+                ciflval.s = clean_string( cif_flex_token(), /* is_textfield = */ 0, cif_cc, ex );
                 return _LOOP_;
-            } else if( starts_with_keyword( "stop_", token ) &&
-                strlen( token ) == 5 ) {
+            } else if( starts_with_keyword( "stop_", cif_flex_token() ) && pos == 5 ) {
                 /* stop field: */
                 ciferror( "STOP_ symbol detected -- "
                          "it is not acceptable in CIF v1.1" );
-            } else if( starts_with_keyword( "global_", token ) &&
-                strlen( token ) == 7 ) {
+            } else if( starts_with_keyword( "global_", cif_flex_token() ) && pos == 7 ) {
                 /* global field: */
                 ciferror( "GLOBAL_ symbol detected -- "
                          "it is not acceptable in CIF v1.1" );
             } else {
-                if( token[0] == '[' ) {
+                if( cif_flex_token()[0] == '[' ) {
                     /* opening bracket is a reserved symbol, unquoted strings
                        may not start with it: */
                     if( !cif_lexer_has_flags
@@ -450,7 +393,7 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                                  "and may not start an unquoted string" );
                     }
                 }
-                if( token[0] == ']' ) {
+                if( cif_flex_token()[0] == ']' ) {
                     /* closing bracket is a reserved symbol, unquoted strings
                        may not start with it: */
                     if( !cif_lexer_has_flags
@@ -459,27 +402,27 @@ static int cif_lexer( FILE *in, cexception_t *ex )
                                  "and may not start an unquoted string" );
                     }
                 }
-                if( token[0] == '$' ) {
+                if( cif_flex_token()[0] == '$' ) {
                     /* dollar is a reserved symbol, unquoted strings
                        may not start with it: */
                     ciferror( "dollar symbol ('$') must not start an "
                              "unquoted string" );
                 }
-                if( token[0] != '[' &&
-                    token[0] != ']' &&
-                    token[0] != '$' ) {
+                if( cif_flex_token()[0] != '[' &&
+                    cif_flex_token()[0] != ']' &&
+                    cif_flex_token()[0] != '$' ) {
                     if( yy_flex_debug ) {
-                        printf( ">>> UQSTRING: '%s'\n", token );
+                        printf( ">>> UQSTRING: '%s'\n", cif_flex_token() );
                     }
-                    ciflval.s = clean_string( token, /* is_textfield = */ 0,
-                                              ex );
+                    ciflval.s = clean_string( cif_flex_token(), /* is_textfield = */ 0,
+                                              cif_cc, ex );
                     return _UQSTRING;
                 } else {
                     if( yy_flex_debug ) {
-                        printf( ">>> SQSTRING (corrected bracket): '%s'\n", token );
+                        printf( ">>> SQSTRING (corrected bracket): '%s'\n", cif_flex_token() );
                     }
-                    ciflval.s = clean_string( token, /* is_textfield = */ 0,
-                                              ex );
+                    ciflval.s = clean_string( cif_flex_token(), /* is_textfield = */ 0,
+                                              cif_cc, ex );
                     return _SQSTRING;
                 }
             }
@@ -488,173 +431,3 @@ static int cif_lexer( FILE *in, cexception_t *ex )
 
     return 0;
 }
-
-static void pushchar( char **buf, size_t *length, size_t pos, int ch )
-{
-    char *str;
-
-    if( !buf || pos >= *length ) {
-        size_t max_size = (size_t)-1;
-        if( *length == 0 ) {
-            *length = 128;
-        } else {
-            if( *length > max_size / 2 ) {
-                cexception_raise( NULL, -99, "cannot double the buffer size" );
-            }
-        }
-        *length *= 2;
-        if( yy_flex_debug ) {
-            printf( ">>> reallocating lex token buffer to %lu\n", *length );
-        }
-        *buf = reallocx( *buf, *length, NULL );
-    }
-
-    str = *buf;
-
-    assert( pos < *length );
-    str[pos] = ch;
-}
-
-void ungetlinec( int ch, FILE *in )
-{
-    ungot_ch = 1;
-    /* CHECKME: see if the lines are switched correctly when '\n' is
-       pushed back at the end of a DOS new line: */
-    if( ch == '\n' || ch == '\r' ) {
-        thisTokenLine = lastTokenLine;
-        currLine --;
-    }
-    ungetc( ch, in );
-}
-
-static int getlinec( FILE *in, cexception_t *ex )
-{
-    int ch = getc( in );
-    static char prevchar;
-
-    if( ch != EOF && !ungot_ch ) {
-        if( ch == '\n' || ch == '\r' ) {
-            if( ch == '\r' || (ch == '\n' && prevchar != '\r' &&
-                               prevchar != '\n')) {
-                prevLine = lineCnt;
-                if( lastTokenLine )
-                    freex( lastTokenLine );
-                if( current_line ) {
-                    lastTokenLine = strdupx( current_line, ex );
-                    if( report_long_items ) {
-                        if( strlen( current_line ) > cif_mandated_line_length ) {
-                            yynote_token( cif_cc, cxprintf( "line exceeds %d characters", 
-                                              cif_mandated_line_length ),
-                                  cif_flex_previous_line_number(), -1, ex );
-                        }
-                    }
-                } else {
-                    lastTokenLine = NULL;
-                }
-            }
-            if( ch == '\r' || (ch == '\n' && prevchar != '\r' )) {
-                lineCnt ++;
-                current_pos = 0;
-            }
-            pushchar( &current_line, &currentl_line_length, 0, '\0' );
-        } else {
-            pushchar( &current_line, &currentl_line_length, current_pos++, ch );
-            pushchar( &current_line, &currentl_line_length, current_pos, '\0' );
-        }
-        prevchar = ch;
-        currentLine = thisTokenLine = current_line;
-        /* printf( ">>> lastTokenLine = '%s'\n", lastTokenLine ); */
-        /* printf( ">>> thisTokenLine = '%s'\n", thisTokenLine ); */
-    }
-    currLine = lineCnt;
-    ungot_ch = 0;
-    return ch;
-}
-
-int cif_flex_current_line_number( void ) { return currLine; }
-int cif_flex_previous_line_number( void ) { return prevLine; }
-void cif_flex_set_current_line_number( ssize_t line ) { lineCnt = line; }
-int cif_flex_current_position( void ) { return thisTokenPos; }
-int cif_flex_previous_position( void ) { return lastTokenPos; }
-void cif_flex_set_current_position( ssize_t pos ) { current_pos = pos - 1; }
-const char *cif_flex_current_line( void ) { return thisTokenLine; }
-const char *cif_flex_previous_line( void ) { return lastTokenLine; }
-
-static char *clean_string( char *src, int is_textfield, cexception_t *ex )
-{
-    int DELTA = 8;
-    ssize_t length = strlen( src );
-    char *volatile new = mallocx( length + 1, ex );
-    char *dest = new;
-    char *start = src;
-    int non_ascii_explained = 0;
-
-    cexception_t inner;
-    cexception_guard( inner ) {
-        while( *src != '\0' ) {
-            if( ( (*src & 255 ) < 32 || (*src & 255 ) == 127 ||
-                ( !cif_lexer_has_flags(CIF_FLEX_LEXER_ALLOW_HIGH_CHARS) &&
-                  (*src & 255 ) > 127 ) )
-                && (*src & 255 ) != '\n'
-                && (*src & 255 ) != '\t'
-                && (*src & 255 ) != '\r' ) {
-                if( cif_lexer_has_flags
-                (CIF_FLEX_LEXER_FIX_NON_ASCII_SYMBOLS)) {
-                    /* Do magic with non-ASCII symbols */
-                    *dest = '\0';
-                    length += DELTA;
-                    new = reallocx( new, length + 1, &inner );
-                    strcat( new, cxprintf( "&#x%04X;", *src & 255 ) );
-                    dest = new + strlen( new ) - 1;
-                    if( non_ascii_explained == 0 ) {
-                        if( is_textfield == 0 ) {
-                            print_message( cif_cc, "WARNING", "non-ASCII symbols "
-                                           "encountered in the text", ":",
-                                           cif_flex_current_line_number(),
-                                           cif_flex_current_position()+1,
-                                           ex );
-                            print_trace( cif_cc, (char*)cif_flex_current_line(),
-                                         cif_flex_current_position()+1, ex );
-                        } else {
-                            print_message( cif_cc, "WARNING", "non-ASCII symbols "
-                                           "encountered in the text field -- "
-                                           "replaced with XML entities", ":",
-                                           cif_flex_current_line_number(),
-                                           -1, ex );
-                            print_current_text_field( cif_cc, start, ex );
-                        }
-                        non_ascii_explained = 1;
-                    }
-                } else {
-                    if( non_ascii_explained == 0 ) {
-                        if( is_textfield == 0 ) {
-                            ciferror( "incorrect CIF syntax" );
-                        } else {
-                            print_message( cif_cc, "ERROR", "non-ASCII symbols "
-                                           "encountered "
-                                           "in the text field", ":",
-                                           cif_flex_current_line_number(),
-                                           -1, ex );
-                            print_current_text_field( cif_cc, start, ex );
-                            cif_compiler_increase_nerrors( cif_cc );
-                        }
-                        non_ascii_explained = 1;
-                    }
-                    dest--; /* Omit non-ASCII symbols */
-                }
-            } else if( (*src & 255) == '\r' ) {
-                dest--; /* Skip carriage return symbols */
-            } else {
-                *dest = *src;
-            }
-            src++;
-            dest++;
-        }
-    }
-    cexception_catch {
-        freex( new );
-        cexception_reraise( inner, ex );
-    }
-    *dest = '\0';
-    return new;
-}
