@@ -1,8 +1,8 @@
 /*---------------------------------------------------------------------------*\
-**$Author: andrius $
-**$Date: 2019-11-25 17:15:16 +0200 (Mon, 25 Nov 2019) $ 
-**$Revision: 7487 $
-**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v2.12/src/components/codcif/common.h $
+**$Author: antanas $
+**$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $ 
+**$Revision: 8230 $
+**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/components/codcif/common.h $
 \*---------------------------------------------------------------------------*/
 
 #ifndef __COMMON_H
@@ -10,12 +10,9 @@
 
 #include <unistd.h>
 #include <math.h>
+#include <cexceptions.h>
+#include <cif_compiler.h>
 
-char *strclone( const char *s );
-char *strnclone( const char *s, size_t length );
-char *strappend( char *s, const char *suffix );
-char *process_escapes( char *str );
-char translate_escape( char **s );
 ssize_t countchars( char c, char *s );
 
 int starts_with_keyword( char *keyword, char *string );
@@ -25,6 +22,7 @@ int is_cif_space( char c );
 
 char *cif_unprefix_textfield( char *tf );
 char *cif_unfold_textfield( char *tf );
+char *clean_string( char *src, int is_textfield, CIF_COMPILER *cif_cc, cexception_t *ex );
 int is_tag_value_unknown( char *tv );
 
 void fprintf_escaped( const char *message,
