@@ -1,8 +1,8 @@
 /*---------------------------------------------------------------------------*\
-**$Author: andrius $
-**$Date: 2019-11-18 13:05:55 +0200 (Mon, 18 Nov 2019) $ 
-**$Revision: 7434 $
-**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v2.12/src/components/codcif/datablock.c $
+**$Author: antanas $
+**$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $ 
+**$Revision: 8230 $
+**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/components/codcif/datablock.c $
 \*---------------------------------------------------------------------------*/
 
 /* representation of the CIF data for the CIF parser. */
@@ -209,7 +209,7 @@ ssize_t *datablock_value_lengths( DATABLOCK *datablock )
 
 CIFVALUE *datablock_cifvalue( DATABLOCK *datablock, int tag_nr, int val_nr )
 {
-    if( tag_nr >= datablock->length ) {
+    if( tag_nr >= (int)datablock->length ) {
         return NULL;
     }
     if( val_nr >= datablock->value_lengths[tag_nr] ) {
@@ -233,13 +233,6 @@ int *datablock_in_loop( DATABLOCK *datablock )
     return datablock->in_loop;
 }
 
-/*
-cif_value_type_t **datablock_types( DATABLOCK *datablock )
-{
-    return datablock->types;
-}
-*/
-
 cif_value_type_t datablock_value_type( DATABLOCK *datablock, int tag_nr, int val_nr )
 {
     CIFVALUE *v = datablock_cifvalue( datablock, tag_nr, val_nr );
@@ -497,7 +490,7 @@ void datablock_finish_loop( DATABLOCK *datablock, cexception_t *ex )
     datablock->loop_first[i] = datablock->loop_start;
     datablock->loop_last[i] = datablock->length - 1;
 
-    for( j = datablock->loop_start; j < datablock->length; j++ ) {
+    for( j = datablock->loop_start; j < (ssize_t)datablock->length; j++ ) {
         datablock->in_loop[j] = i;
     }
 
@@ -510,8 +503,8 @@ void datablock_push_loop_cifvalue( DATABLOCK * datablock, CIFVALUE *value,
     cexception_t inner;
     ssize_t i, j, capacity;
 
-    assert( datablock->loop_start < datablock->length );
-    assert( datablock->loop_current < datablock->length );
+    assert( datablock->loop_start < (ssize_t)datablock->length );
+    assert( datablock->loop_current < (ssize_t)datablock->length );
 
     cexception_guard( inner ) {
         i = datablock->loop_current;
@@ -542,7 +535,7 @@ void datablock_push_loop_cifvalue( DATABLOCK * datablock, CIFVALUE *value,
         datablock->value_lengths[i] = j + 1;
         datablock->values[i][j] = value;
         datablock->loop_current++;
-        if( datablock->loop_current >= datablock->length ) {
+        if( datablock->loop_current >= (ssize_t)datablock->length ) {
             datablock->loop_current = datablock->loop_start;
         }
     }
