@@ -2,7 +2,7 @@
 **$Author: antanas $
 **$Date: 2020-01-23 08:18:43 +0200 (Thu, 23 Jan 2020) $ 
 **$Revision: 7669 $
-**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v2.12/src/components/codcif/programs/cif_list_tags.c $
+**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/components/codcif/programs/cif_list_tags.c $
 \*---------------------------------------------------------------------------*/
 
 /* uses: */
@@ -18,7 +18,7 @@
 #include <cxprintf.h>
 #include <version.h>
 
-static char *source_URL = "$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v2.12/src/components/codcif/programs/cif_list_tags.c $";
+static char *source_URL = "$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/components/codcif/programs/cif_list_tags.c $";
 
 static char *usage_text[2] = {
 
