@@ -1,8 +1,8 @@
 /*---------------------------------------------------------------------------*\
 **$Author: antanas $
-**$Date: 2019-12-06 13:49:15 +0200 (Fri, 06 Dec 2019) $ 
-**$Revision: 7553 $
-**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v2.12/src/components/codcif/programs/cifparse.c $
+**$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $ 
+**$Revision: 8230 $
+**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/components/codcif/programs/cifparse.c $
 \*---------------------------------------------------------------------------*/
 
 /* uses: */
@@ -15,13 +15,14 @@
 #include <cif_compiler.h>
 #include <cif_grammar_y.h>
 #include <cif_grammar_flex.h>
+#include <cif_lex_buffer.h>
 #include <cif_lexer.h>
 #include <allocx.h>
 #include <cxprintf.h>
 #include <ctype.h>
 #include <version.h>
 
-static char *source_URL = "$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v2.12/src/components/codcif/programs/cifparse.c $";
+static char *source_URL = "$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/components/codcif/programs/cifparse.c $";
 
 static char *usage_text[2] = {
 
@@ -239,7 +240,8 @@ int main( int argc, char *argv[], char *env[] )
   }
 
   if( report_long_items.value.b == 1 ) {
-      cif_lexer_set_report_long_items( 1 );
+      cif_lexer_set_report_long_lines( 1 );
+      cif_lexer_set_report_long_tags( 1 );
   }
 
   if( line_length_limit.present ) {
