@@ -1,8 +1,8 @@
 #--*-perl-*-------------------------------------------------------------
-#$Author: saulius $
-#$Date: 2020-06-19 16:43:25 +0300 (Fri, 19 Jun 2020) $
-#$Revision: 8051 $
-#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v2.12/src/lib/perl5/COD/Algebra/GaussJordanBigRat.pm $
+#$Author: antanas $
+#$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $
+#$Revision: 8230 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/Algebra/GaussJordanBigRat.pm $
 #-----------------------------------------------------------------------
 #*
 # Performs Gauss-Jordan elimination on a matrix; uses BigRat arithmetic.
@@ -20,7 +20,6 @@ our @EXPORT_OK = qw(
     gj_elimination_non_zero_elements
     forward_elimination
     back_substitution
-    backward_elimination
 );
 
 # Run the G-J elimination process, return a reduced echelon form
@@ -130,13 +129,6 @@ sub forward_elimination
     return \@m;
 }
 
-# The 'backward_elimination' name is deprecated and retained only for
-# compatibility
-sub backward_elimination
-{
-    return &back_substitution
-}
-
 # Subtract one row (a vector) multiplied by a coefficient from another
 # row:
 sub v_k_sub
