@@ -1,8 +1,8 @@
 #--*-perl-*-------------------------------------------------------------
-#$Author: antanas $
-#$Date: 2015-11-16 16:04:37 +0200 (Mon, 16 Nov 2015) $
-#$Revision: 4287 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/Algebra/JacobiEigen.pm $
+#$Author: andrius $
+#$Date: 2020-07-07 13:48:39 +0300 (Tue, 07 Jul 2020) $
+#$Revision: 8111 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/Algebra/JacobiEigen.pm $
 #-----------------------------------------------------------------------
 # Find all eigenvalues and eigenvectors of a symmetric matrix 'a' of order n.
 # using the Jacobi method. Implemented as described in 
@@ -54,9 +54,9 @@ sub jacobi_eigenvv ## ( @ )
             return ( \@eigenvectors, \@eigenvalues, $sweep_count );
         }
 
-        my $tresh = ( $sweep_count < 4 ) ? 0.2 * $sum/($n*$n) : 0.0;
+        my $thresh = ( $sweep_count < 4 ) ? 0.2 * $sum/($n*$n) : 0.0;
 
-        &jacobi_sweep( \@a, \@eigenvalues, \@eigenvectors, $sweep_count, $tresh );
+        &jacobi_sweep( \@a, \@eigenvalues, \@eigenvectors, $sweep_count, $thresh );
     }
     die "ERROR, too many iterations in jacobi_eigenvv()\n";
 }
@@ -88,7 +88,7 @@ sub jacobi_transform($$$$$$$)
 
 sub jacobi_sweep( $$$$$ )
 {
-    my ( $a, $d, $v, $sweep, $tresh ) = @_;
+    my ( $a, $d, $v, $sweep, $thresh ) = @_;
 
     my $n = scalar(@{$a});
     my $b = [ map { $_ } @{$d} ];
@@ -100,7 +100,7 @@ sub jacobi_sweep( $$$$$ )
             if ($sweep > 4 && $g <= $eps*abs($d->[$p]) && 
                               $g <= $eps*abs($d->[$q]) ) {
                 $a->[$p][$q] = 0.0;
-            } elsif (abs($a->[$p][$q]) > $tresh) {
+            } elsif (abs($a->[$p][$q]) > $thresh) {
                 my $h = $d->[$q] - $d->[$p];
                 my $t;
                 if ( $g <= $eps*abs($h)) {
