@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
-#$Author: andrius $
-#$Date: 2019-11-19 09:15:03 +0200 (Tue, 19 Nov 2019) $ 
-#$Revision: 7443 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/AtomNeighbours.pm $
+#$Author: antanas $
+#$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $ 
+#$Revision: 8230 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/AtomNeighbours.pm $
 #------------------------------------------------------------------------------
 #*
 #  Build a list of neighbours for each atom from an atom list.
@@ -16,7 +16,9 @@ use Carp qw( croak );
 use COD::AtomBricks qw( build_bricks get_atom_index get_search_span );
 use COD::AtomProperties;
 use COD::Algebra::Vector qw( distance );
-use COD::CIF::Data::AtomList qw( atoms_are_alternative );
+use COD::CIF::Data::AtomList qw( atoms_are_alternative
+                                 datablock_from_atom_array );
+use COD::CIF::Tags::Manage qw( set_loop_tag );
 
 require Exporter;
 our @ISA = qw( Exporter );
@@ -25,7 +27,8 @@ our @EXPORT_OK = qw(
     get_max_vdw_radius
     make_neighbour_list
     neighbour_list_from_chemistry_mol
-    neighbour_list_from_chemistry_openbabel_obmol
+    neighbour_list_to_cif_datablock
+    neighbour_list_to_graph
 );
 
 #==============================================================================#
@@ -346,84 +349,58 @@ sub neighbour_list_from_chemistry_mol
 }
 
 #==============================================================================
-# Generates neighbour list from Chemistry::OpenBabel::OBMol object.
-# Tested with the version 2.2.3 of OpenBabel.
-# This method is deprecated and will be removed in future releases.
-sub neighbour_list_from_chemistry_openbabel_obmol
+# Generates CIF datablock from neighbour list. Bonds are represented in
+# _geom_bond_* CIF data items.
+sub neighbour_list_to_cif_datablock
 {
-    my( $obmol ) = @_;
+    my( $neighbour_list ) = @_;
 
-    my %neighbour_list = (
-        atoms => [],
-        neighbours => [],
-    );
+    my $datablock = datablock_from_atom_array( $neighbour_list->{atoms} );
+    my @labels1 = map { $neighbour_list->{atoms}[$_]{name} }
+                  map { ( $_ ) x scalar @{$neighbour_list->{neighbours}[$_]} }
+                      0..$#{$neighbour_list->{neighbours}};
+    my @labels2 = map { $neighbour_list->{atoms}[$_]{name} }
+                  map { @{$neighbour_list->{neighbours}[$_]} }
+                      0..$#{$neighbour_list->{neighbours}};
 
-    my %atom_ids;
-    for my $i (1..$obmol->NumAtoms()) { # Atoms are indexed from 1
-        my $atom = $obmol->GetAtom($i);
-        my %atom_info;
+    set_loop_tag( $datablock,
+                  '_geom_bond_atom_site_label_1',
+                  '_geom_bond_atom_site_label_1',
+                  [ map  { $labels1[$_] }
+                    grep { $labels1[$_] le $labels2[$_] }
+                         0..$#labels1 ] );
+    set_loop_tag( $datablock,
+                  '_geom_bond_atom_site_label_2',
+                  '_geom_bond_atom_site_label_1',
+                  [ map  { $labels2[$_] }
+                    grep { $labels1[$_] le $labels2[$_] }
+                         0..$#labels2 ] );
+    return $datablock;
+}
 
-        my( $type ) = sort
-                      grep { $COD::AtomProperties::atoms{$_}->{atomic_number} ==
-                             $atom->GetAtomicNum() }
-                      keys %COD::AtomProperties::atoms;
+#==============================================================================
+# Generates Graph::Undirected object from neighbour list
+sub neighbour_list_to_graph
+{
+    my( $neighbour_list ) = @_;
 
-        # Since the atomic numbers of H and D are equal, chemical type
-        # has to be checked:
-        if( $type eq 'D' && $atom->GetType() =~ /^H/ ) {
-            $type = 'H';
-        }
+    require Graph::Undirected;
 
-        $atom_info{"name"}                  = $type . $i;
-        $atom_info{"site_label"}            = $type . $i;
-        $atom_info{"cell_label"}            = $type . $i;
-        $atom_info{"index"}                 = $i-1;
-        $atom_info{"symop"}                 =
-          [
-            [ 1, 0, 0, 0 ],
-            [ 0, 1, 0, 0 ],
-            [ 0, 0, 1, 0 ],
-            [ 0, 0, 0, 1 ]
-          ];
-        $atom_info{"symop_id"}              = 1;
-        $atom_info{"unity_matrix_applied"}  = 1;
-        $atom_info{"translation_id"}        = "555";
-        $atom_info{"translation"}           = [ 0, 0, 0 ];
-
-        $atom_info{"chemical_type"}         = $type;
-        $atom_info{"assembly"}              = ".";
-        $atom_info{"group"}                 = ".";
-        $atom_info{"atom_site_occupancy"}   = 1;
-        $atom_info{"attached_hydrogens"}    = $atom->ImplicitHydrogenCount();
-
-        $atom_ids{$atom->GetIdx()} = $i-1;
-        push( @{$neighbour_list{atoms}}, \%atom_info );
-    }
-
-    for my $i (0..$obmol->NumBonds()-1) { # Bonds are indexed from 0 (?)
-        my $bond = $obmol->GetBond($i);
-        push @{$neighbour_list{neighbours}
-                              [$atom_ids{$bond->GetBeginAtom()->GetIdx()}]},
-             $atom_ids{$bond->GetEndAtom()->GetIdx()};
-        push @{$neighbour_list{neighbours}
-                              [$atom_ids{$bond->GetEndAtom()->GetIdx()}]},
-             $atom_ids{$bond->GetBeginAtom()->GetIdx()};
-    }
-
-    # Aromatic atoms are considered planar only if they have three
-    # or more neighbours, as any three points lie on the same
-    # plane.
-    for my $i (1..$obmol->NumAtoms()) { # Atoms are indexed from 1
-        my $atom = $obmol->GetAtom($i);
-        
-        if( $atom->IsAromatic() &&
-            @{$neighbour_list{neighbours}->[$i-1]} +
-            $neighbour_list{atoms}->[$i-1]{"attached_hydrogens"} >= 3 ) {
-            $neighbour_list{atoms}->[$i-1]{"planarity"} = 0;
+    my $graph = Graph::Undirected->new;
+    for my $atom (@{$neighbour_list->{atoms}}) {
+        my $index1 = $atom->{index};
+        if( @{$neighbour_list->{neighbours}} > $index1 &&
+            @{$neighbour_list->{neighbours}[$index1]} ) {
+            for my $index2 (@{$neighbour_list->{neighbours}[$index1]}) {
+                $graph->add_edge( $neighbour_list->{atoms}[$index1],
+                                  $neighbour_list->{atoms}[$index2] );
+            }
+        } else {
+            $graph->add_vertex( $atom );
         }
     }
 
-    return \%neighbour_list;
+    return $graph;
 }
 
 1;
