@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
 #$Author: antanas $
-#$Date: 2019-04-23 01:57:49 +0300 (Tue, 23 Apr 2019) $ 
-#$Revision: 6975 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/CIF/DDL/Ranges.pm $
+#$Date: 2020-07-05 05:29:44 +0300 (Sun, 05 Jul 2020) $
+#$Revision: 8103 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/CIF/DDL/Ranges.pm $
 #------------------------------------------------------------------------------
 #*
 #* A set of subroutines for handling ranges as defined in the Dictionary
@@ -47,7 +47,7 @@ sub parse_range
 
 ##
 # Construct a string representation of a range from the range array as
-# returned by the COD::CIF::DDL::Range::parse_range subroutine().
+# returned by the COD::CIF::DDL::Ranges::parse_range() subroutine.
 # The returned string does not have the same format as the range string
 # found in DDL files and is mainly meant to be used in audit messages.
 #
@@ -56,10 +56,10 @@ sub parse_range
 #       COD::CIF::DDL::Ranges::parse_range() subroutine.
 # @param $options
 #       Reference to a hash of options. The following options are recognised:
-#         {
+#       {
 #         # String representing the type of value ('numb' or 'char')
 #           'type'  => 'char',
-#         }
+#       }
 # @return $string
 #       A string representing the enumeration range.
 ##
@@ -81,7 +81,7 @@ sub range_to_string
 
 ##
 # Construct a string representation of a numeric range from the range array
-# as returned by the COD::CIF::DDL::Range::parse_range subroutine().
+# as returned by the COD::CIF::DDL::Ranges::parse_range() subroutine.
 # The returned string does not have the same format as the range string
 # found in DDL files and is mainly meant to be used in audit messages.
 #
@@ -105,7 +105,7 @@ sub range_to_string_numeric
 
 ##
 # Construct a string representation of a character range from the range array
-# as returned by the COD::CIF::DDL::Range::parse_range subroutine().
+# as returned by the COD::CIF::DDL::Ranges::parse_range() subroutine.
 # The returned string does not have the same format as the range string
 # found in DDL files and is mainly meant to be used in audit messages.
 #
@@ -120,9 +120,9 @@ sub range_to_string_char
     my ($range) = @_;
 
     my $string;
-    $string  = (defined $range->[0] ? "[$range->[0]" : '[<any>');
+    $string  = (defined $range->[0] ? "['$range->[0]'" : '[<any>');
     $string .= ', ';
-    $string .= (defined $range->[1] ? "$range->[1]]" : '<any>]');
+    $string .= (defined $range->[1] ? "'$range->[1]']" : '<any>]');
 
     return $string;
 }
@@ -134,16 +134,23 @@ sub range_to_string_char
 #       Value to be checked.
 # @param $param
 #       Parameter hash with the following keys:
-#         {
+#       {
+#         # String, representing the type of value ('numb' or 'char').
+#         # Default: 'char'
 #           'type'  => 'char',
-#                String, representing the type of value ('numb' or 'char').
-#           'range' => [0, 10]
-#                Reference to a range array as returned by the
-#                COD::CIF::DDL::Ranges::parse_range() subroutine.
-#           'sigma  => 0.1
-#               Standard deviation to be used when comparing numeric values
-#               (3 sigma rule). If sigma is not provided, values are compared
-#               disregarding the standard deviation.
+#         # Reference to a range array as returned by the
+#         # COD::CIF::DDL::Ranges::parse_range() subroutine
+#           'range' => [0, 10],
+#         # Standard uncertainty (s.u.) to be used when determining
+#         # if a numeric value resides in the specified range.
+#         # See the is_in_range_numeric() subroutine for more details
+#           'sigma' => 0.1,
+#         # Multiplier that should be applied to the standard
+#         # uncertainty (s.u.) when determining if a numeric
+#         # value resides in the specified range. See the
+#         # is_in_range_numeric() subroutine for more details.
+#         # Default: 3
+#           'multiplier' => 3,
 #         }
 # @return
 #       -1 if no ranges were provided for the value;
@@ -157,6 +164,8 @@ sub is_in_range
     my $range = $param->{'range'};
     my $type  = defined $param->{'type'} ?
                         $param->{'type'} : 'char';
+    my $su_multiplier = defined $param->{'multiplier'} ?
+                                $param->{'multiplier'} : 3;
 
     if( !defined $range->[0] &&
         !defined $range->[1] ) {
@@ -164,7 +173,14 @@ sub is_in_range
     }
 
     if( $type eq 'numb' ) {
-        return is_in_range_numeric( $value, $param );
+        return is_in_range_numeric(
+                    $value,
+                    {
+                        'range' => $range,
+                        'sigma' => $param->{'sigma'},
+                        'multiplier' => $su_multiplier,
+                    }
+                );
     } else {
         return is_in_range_char( $value, $param );
     }
@@ -172,19 +188,27 @@ sub is_in_range
 
 ##
 # Checks numeric value against an inclusive numeric range.
+#
 # @param $value
 #       Value to be checked.
 # @param $param
 #       Parameter hash with the following keys:
-#         {
-#           'range' => [0, 10]
-#                Reference to a range array as returned by the
-#                COD::CIF::DDL::Ranges::parse_range() subroutine.
-#           'sigma  => 0.1
-#               Standard deviation to be used when comparing numeric values
-#               (3 sigma rule). If sigma is not provided, values are compared
-#               disregarding the standard deviation.
-#         }
+#       {
+#         # Reference to a range array as returned by the
+#         # COD::CIF::DDL::Ranges::parse_range() subroutine
+#           'range' => [0, 10],
+#         # Standard uncertainty (s.u.) to be used when determining
+#         # if a numeric value resides in the specified range.
+#         # See the 'multiplier' option for more details
+#           'sigma'  => 0.1,
+#         # Multiplier that should be applied to the standard
+#         # uncertainty (s.u.) when determining if a numeric
+#         # value resides in the specified range. For example,
+#         # a multiplier of 3.5 means that the value is treated
+#         # as valid if it falls in the interval of
+#         # [lower bound - 3.5 * s.u.; upper bound + 3.5 * s.u.]
+#           'multiplier' => 3,
+#       }
 # @return
 #        0 if the value is out of the provided range or is not a number
 #          at all;
@@ -197,14 +221,15 @@ sub is_in_range_numeric
     my $min   = $param->{'range'}[0];
     my $max   = $param->{'range'}[1];
     my $sigma = $param->{'sigma'};
+    my $multiplier = $param->{'multiplier'};
 
     if( ! looks_like_number($value) ) {
         return 0;
     }
 
-    if( defined $sigma ) {
-        $min = $min - 3 * $sigma if defined $min;
-        $max = $max + 3 * $sigma if defined $max;
+    if( defined $sigma && defined $multiplier ) {
+        $min = $min - $multiplier * $sigma if defined $min;
+        $max = $max + $multiplier * $sigma if defined $max;
     };
 
     if(
@@ -223,13 +248,13 @@ sub is_in_range_numeric
 #       Value to be checked.
 # @param $param
 #       Parameter hash with the following keys:
-#         {
-#           'range' => [a, c]
-#                Reference to a range array as returned by the
-#                COD::CIF::DDL::Ranges::parse_range() subroutine.
-#         }
+#       {
+#         # Reference to a range array as returned by the
+#         # COD::CIF::DDL::Ranges::parse_range() subroutine.
+#           'range' => ['a', 'c'],
+#       }
 # @return
-#        0 if the value is out of the provided range;
+#        0 if the value is out of the provided range,
 #        1 if the value is in the provided range.
 ##
 sub is_in_range_char
