@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
 #$Author: antanas $
-#$Date: 2019-01-02 15:52:23 +0200 (Wed, 02 Jan 2019) $ 
-#$Revision: 6605 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/CIF/DDL/Validate.pm $
+#$Date: 2020-07-09 15:18:42 +0300 (Thu, 09 Jul 2020) $ 
+#$Revision: 8152 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/CIF/DDL/Validate.pm $
 #------------------------------------------------------------------------------
 #*
 #* A set of subroutines used to validate CIF files against ontologies expressed
@@ -14,9 +14,12 @@ package COD::CIF::DDL::Validate;
 use strict;
 use warnings;
 
+use COD::CIF::Tags::CanonicalNames qw( canonical_tag_name );
+
 require Exporter;
 our @ISA = qw( Exporter );
 our @EXPORT_OK = qw(
+    canonicalise_tag
     check_enumeration_set
 );
 
@@ -43,13 +46,7 @@ sub check_enumeration_set
 {
     my ($values, $enum_set, $options) = @_;
 
-    my @validation_messages;
-
-    my $ignore_case  = $options->{'ignore_case'};
-    my $treat_as_set = $options->{'treat_as_set'};
-
     my $enum_regex = build_enum_regex( $enum_set, $options );
-
     my @is_proper_enum = map { $_ !~ m/$enum_regex/s } @{$values};
 
     return \@is_proper_enum;
@@ -91,4 +88,19 @@ sub build_enum_regex
     return $regex;
 }
 
+##
+# Canonicalises a CIF data name to a standard form used in validation messages.
+#
+# @param $tag
+#       Data name that should be canonicalised.
+# @return
+#       Canonicalised data name.
+##
+sub canonicalise_tag
+{
+    my ($tag) = @_;
+
+    return canonical_tag_name($tag);
+}
+
 1;
