@@ -1,8 +1,8 @@
- #------------------------------------------------------------------------------
-#$Author: antanas $
-#$Date: 2020-01-01 13:51:08 +0200 (Wed, 01 Jan 2020) $ 
-#$Revision: 7638 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/CIF/Data/AtomList.pm $
+#------------------------------------------------------------------------------
+#$Author: andrius $
+#$Date: 2020-07-07 16:26:28 +0300 (Tue, 07 Jul 2020) $ 
+#$Revision: 8121 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/CIF/Data/AtomList.pm $
 #------------------------------------------------------------------------------
 #*
 #  Construct a common data structure, representing list of atoms in
@@ -553,8 +553,9 @@ sub datablock_from_atom_array
 {
     my( $atoms ) = @_;
 
-    my $has_disorder = grep { $_->{group} ne '.' ||
-                              $_->{assembly} ne '.' } @$atoms;
+    my $has_disorder =
+        grep { (defined $_->{group}    && $_->{group}    ne '.') ||
+               (defined $_->{assembly} && $_->{assembly} ne '.') } @$atoms;
     my $has_attached_hydrogens = grep { $_->{attached_hydrogens} } @$atoms;
 
     my %has_key;
@@ -579,12 +580,24 @@ sub datablock_from_atom_array
                   '_atom_site_label',
                   [ map { $_->{atom_site_type_symbol} } @$atoms ] )
         if $has_key{atom_site_type_symbol};
-    set_loop_tag( $datablock, '_atom_site_fract_x', '_atom_site_label',
-                  [ map { $_->{coordinates_fract}[0] } @$atoms ] );
-    set_loop_tag( $datablock, '_atom_site_fract_y', '_atom_site_label',
-                  [ map { $_->{coordinates_fract}[1] } @$atoms ] );
-    set_loop_tag( $datablock, '_atom_site_fract_z', '_atom_site_label',
-                  [ map { $_->{coordinates_fract}[2] } @$atoms ] );
+    set_loop_tag( $datablock,
+                  '_atom_site_fract_x',
+                  '_atom_site_label',
+                  [ map { defined $_->{coordinates_fract}[0]
+                                ? $_->{coordinates_fract}[0] : '?'}
+                        @$atoms ] );
+    set_loop_tag( $datablock,
+                  '_atom_site_fract_y',
+                  '_atom_site_label',
+                  [ map { defined $_->{coordinates_fract}[1]
+                                ? $_->{coordinates_fract}[1] : '?'}
+                        @$atoms ] );
+    set_loop_tag( $datablock,
+                  '_atom_site_fract_z',
+                  '_atom_site_label',
+                  [ map { defined $_->{coordinates_fract}[2]
+                                ? $_->{coordinates_fract}[2] : '?'}
+                        @$atoms ] );
     set_loop_tag( $datablock,
                   '_atom_site_U_iso_or_equiv',
                   '_atom_site_label',
