@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
 #$Author: antanas $
-#$Date: 2020-01-16 16:26:50 +0200 (Thu, 16 Jan 2020) $
-#$Revision: 7660 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/CIF/Data/CIF2COD.pm $
+#$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $
+#$Revision: 8230 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/CIF/Data/CIF2COD.pm $
 #------------------------------------------------------------------------------
 #*
 #  Parse a CIF file, prepare a COD database table entry from it.
@@ -31,65 +31,11 @@ our @EXPORT_OK = qw(
     cif2cod
     validate_SQL_types
     @default_data_fields
-    @new_data_fields
 );
 
 # The default sql table data field that was taken from the
 # cod-add-data.sh script.
 our @default_data_fields = qw (
-    file
-    a
-    siga
-    b
-    sigb
-    c
-    sigc
-    alpha
-    sigalpha
-    beta
-    sigbeta
-    gamma
-    siggamma
-    vol
-    sigvol
-    celltemp
-    sigcelltemp
-    diffrtemp
-    sigdiffrtemp
-    cellpressure
-    sigcellpressure
-    diffrpressure
-    sigdiffrpressure
-    thermalhist
-    pressurehist
-    nel
-    sg
-    sgHall
-    commonname
-    chemname
-    formula
-    calcformula
-    acce_code
-    authors
-    title
-    journal
-    year
-    volume
-    issue
-    firstpage
-    lastpage
-    duplicateof
-    optimal
-    status
-    flags
-    text
-);
-
-my @data_fields = @default_data_fields;
-
-# New sql table data fields.
-
-our @new_data_fields = qw (
     file
     a
     siga
@@ -159,6 +105,7 @@ our @new_data_fields = qw (
     gofall
     gofobs
     gofgt
+    gofref
 
     duplicateof
     optimal
@@ -168,6 +115,8 @@ our @new_data_fields = qw (
     onhold
 );
 
+my @data_fields = @default_data_fields;
+
 # A hash of text fields that do no require specific processing
 # and can be taken directly from the associated data items
 my %text_value_fields2tags = (
@@ -229,9 +178,13 @@ my %num_value_fields2tags = (
     'RFsqd'         => [ qw( _refine_ls_R_Fsqd_factor ) ],
     'RI'            => [ qw( _refine_ls_R_I_factor ) ],
     'gofall'        => [ qw( _refine_ls_goodness_of_fit_all ) ],
-    'gofobs'        => [ qw( _refine_ls_goodness_of_fit_ref
+    # NOTE: 'gofobs' and 'gofgt' intentionally store the same data.
+    # One of these columns will be removed in the future
+    'gofobs'        => [ qw( _refine_ls_goodness_of_fit_gt
                              _refine_ls_goodness_of_fit_obs ) ],
-    'gofgt'         => [ qw( _refine_ls_goodness_of_fit_gt )],
+    'gofgt'         => [ qw( _refine_ls_goodness_of_fit_gt
+                             _refine_ls_goodness_of_fit_obs ) ],
+    'gofref'        => [ qw( _refine_ls_goodness_of_fit_ref ) ],
 );
 
 # A hash of s.u. fields that do no require specific processing
@@ -275,8 +228,8 @@ my %space_groups = map {
 #       # Include the implicit hydrogen atoms into the summary chemical formula.
 #       # The implicit hydrogen atoms are specified using
 #       # the _atom_site_attached_hydrogens data item. 
-#           'use_attached_hydrogens' => 0,
-#       # Boolean value. Default '0'.
+#           'use_attached_hydrogens' => 1,
+#       # Boolean value. Default '1'.
 #       # Accept data blocks without fractional coordinates.
 #           'use_datablocks_without_coord' => 0,
 #       # Boolean value. Default '0'.
@@ -304,7 +257,7 @@ sub cif2cod
                    $options->{'use_datablocks_without_coord'} : 0;
     my $use_attached_hydrogens =
             exists $options->{'use_attached_hydrogens'} ?
-                   $options->{'use_attached_hydrogens'} : 0;
+                   $options->{'use_attached_hydrogens'} : 1;
     my $reformat_space_group =
             exists $options->{'reformat_space_group'} ?
                    $options->{'reformat_space_group'} : 0;
