@@ -1,8 +1,8 @@
 #--*-perl-*-------------------------------------------------------------
-#$Author: antanas $
-#$Date: 2019-12-17 11:46:57 +0200 (Tue, 17 Dec 2019) $
-#$Revision: 7614 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/CIF/Data/CODFlags.pm $
+#$Author: andrius $
+#$Date: 2020-07-07 11:45:07 +0300 (Tue, 07 Jul 2020) $
+#$Revision: 8109 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/CIF/Data/CODFlags.pm $
 #-----------------------------------------------------------------------
 # Test dataset on various criteria.
 #-----------------------------------------------------------------------
@@ -11,7 +11,11 @@ package COD::CIF::Data::CODFlags;
 
 use strict;
 use warnings;
-use COD::CIF::Tags::Manage qw( tag_is_empty );
+use COD::CIF::Tags::Manage qw(
+    contains_data_item
+    has_special_value
+    tag_is_empty
+);
 
 require Exporter;
 our @ISA = qw( Exporter );
@@ -24,6 +28,7 @@ our @EXPORT_OK = qw(
     is_theoretical
     has_coordinates
     has_hkl
+    has_partially_occupied_ordered_atoms
     has_powder_diffraction_intensities
     has_twin_hkl
     has_Fobs
@@ -251,6 +256,25 @@ sub has_hkl($)
     return 1;
 }
 
+sub has_partially_occupied_ordered_atoms($)
+{
+    my ($data_block) = @_;
+
+    my $values = $data_block->{'values'};
+
+    return 0 if tag_is_empty($data_block, '_atom_site_occupancy') ||
+                !contains_data_item($data_block, '_atom_site_disorder_group');
+
+    for my $i (0..$#{$data_block->{'values'}{'_atom_site_label'}}) {
+        next if has_special_value($data_block, '_atom_site_occupancy', $i);
+        next if has_special_value($data_block, '_atom_site_disorder_group', $i);
+        next if $values->{'_atom_site_occupancy'}[$i] == 1;
+        return 1;
+    }
+
+    return 0;
+}
+
 sub has_powder_diffraction_intensities($)
 {
     my ($data_block) = @_;
