@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
 #$Author: antanas $
-#$Date: 2020-01-01 08:42:18 +0200 (Wed, 01 Jan 2020) $
-#$Revision: 7637 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/CIF/Data/CODNumbers.pm $
+#$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $
+#$Revision: 8230 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/CIF/Data/CODNumbers.pm $
 #------------------------------------------------------------------------------
 #*
 #  Find COD numbers of duplicate structures.
@@ -37,10 +37,13 @@ our @EXPORT_OK = qw(
     have_equiv_timestamps
 );
 
-my %default_options = (
+my %DEFAULT_OPTIONS = (
     'check_bibliography'    => 1,
     'check_sample_history'  => 0,
     'check_compound_source' => 0,
+    'use_su'                => 1,
+    'max_cell_length_diff'  => 0.5,
+    'max_cell_angle_diff'   => 1.2,
 );
 
 my $db_fields2tags = {
@@ -117,24 +120,22 @@ my $db_fields2tags = {
 #         # Each duplicate entry is identified by its database ID (i.e. COD ID)
 #         # and described by a data structure as returned by the
 #         # 'get_database_entries()' subroutine:
-#           'duplicates' =>
-#               {
-#                 '1100003' =>
-#                  {
-#                    'id'           => '1100003',
-#                    'filename'     => '1100003',
-#                    'bibliography' => { ... },
-#                    'cell'         => { ... },
-#                    'sigcell'      => { ... },
-#                    'pressure'     => { ... },
-#                    'temperature'  => { ... },
-#                    'history'      => { ... },
-#                    'source'       => { ... },
+#           'duplicates' => {
+#               '1100003' => {
+#                   'id'           => '1100003',
+#                   'filename'     => '1100003',
+#                   'bibliography' => { ... },
+#                   'cell'         => { ... },
+#                   'sigcell'      => { ... },
+#                   'pressure'     => { ... },
+#                   'temperature'  => { ... },
+#                   'history'      => { ... },
+#                   'source'       => { ... },
 #                    ...
-#                  },
-#                  ...
-#               }
+#               },
+#               ...
 #           }
+#       }
 ##
 sub fetch_duplicates_from_database
 {
@@ -142,9 +143,9 @@ sub fetch_duplicates_from_database
 
     $user_options = {} if !defined $user_options;
     my %options = %{$user_options};
-    foreach my $key (keys %default_options) {
+    foreach my $key (keys %DEFAULT_OPTIONS) {
         if ( !defined $options{$key} ) {
-            $options{$key} = $default_options{$key};
+            $options{$key} = $DEFAULT_OPTIONS{$key};
         }
     };
 
@@ -220,23 +221,151 @@ sub fetch_duplicates
 
 #------------------------------------------------------------------------------
 
+##
+# Builds a crystal structure description that contains sufficient information
+# to determine if the given structure is a duplicate of another structure
+# according to the COD criteria [1].
+#
+# @source [1]
+#       https://wiki.crystallography.net/duplicate_entry
+#
+# @param $dataset
+#       Reference to a data block as returned by the COD::CIF::Parser.
+# @param $file
+#       Name of the parsed CIF file that contains the structure.
+# @param $index
+#       Arbitrary index that is used by the fetch_duplicates() subroutine
+#       to order entries while searching for duplicates.
+# @param $options
+#       Reference to a hash of options. The following options are recognised:
+#       {
+#         # Logical values denoting if the number of implicit hydrogen
+#         # atoms should be included when calculating the summary
+#         # chemical formula. Implicit hydrogen atoms are specified
+#         # using the _atom_site_attached_hydrogens data item
+#         # Default: 0
+#           'use_attached_hydrogens' => 0,
+#       }
+# @return
+#       Reference to a data structure of the following form:
+#       {
+#         # Data block code of the CIF data block that contains the structure
+#           'id' => '1000000',
+#         # Name of the parsed CIF file that contains the structure
+#           'filename' => '1000000.cif',
+#         # Arbitrary index that is used by the fetch_duplicates()
+#         # subroutine to order entries while searching for duplicates
+#           'index' => 0,
+#         # Summary chemical formula of the compound as
+#         # explicitly provided in the original file
+#           'chemical_formula_sum' => 'H2 O',
+#         # Summary chemical formula of the compound
+#         # calculated from the crystallographic data
+#           'calc_formula' => 'H2 O',
+#         # Summaary chemical formula of the crystal cell
+#         # calculated from the crystallographic data (Z = 1)
+#           'cell_formula' => 'H8 O4',
+#         # Crystal cell parameter values without
+#         # the standard uncertainty values (see 'sigcell)
+#           'cell' => {
+#               '_cell_length_a' => 7.53,
+#               '_cell_length_b' => 7.53,
+#               '_cell_length_c' => 7.53
+#               '_cell_angle_alpha' => 90,
+#               '_cell_angle_beta' => 90,
+#               '_cell_angle_gamma' => 90,
+#           },
+#         # Standard uncertainty values of crystal cell
+#         # parameter values (see 'cell)
+#           'sigcell' => {
+#               '_cell_length_a' => 2,
+#               '_cell_length_b' => 3,
+#               '_cell_length_c' => 4,
+#               '_cell_angle_alpha' => 1,
+#               '_cell_angle_beta'  => 1,
+#               '_cell_angle_gamma' => 1,
+#           },
+#         # Bibliographical references to the original
+#         # publication describing the structure
+#           'bibliography' => {
+#                '_journal_name_full' => 'Journal',
+#                '_journal_year' => '2000',
+#                '_journal_volume' => '5',
+#                '_journal_issue' => '6',
+#                '_journal_page_first' => '1',
+#                '_journal_page_last' => '2',
+#                '_journal_paper_doi' => '10.1186/s13321-018-0279-6',
+#           },
+#         # Sample preparation details as provided in the original file
+#           'history' => {
+#                '_exptl_crystal_pressure_history' => 'Held under pressure',
+#                '_exptl_crystal_thermal_history' => 'Treated in flames',
+#           },
+#         # Pressure under which the crystal was measured
+#           'pressure' => {
+#                '_cell_measurement_pressure' => 101,
+#                '_cell_measurement_pressure_gPa' => ...,
+#                '_cell_wave_vectors_pressure_max' => ...,
+#                '_cell_wave_vectors_pressure_min' => ...,
+#                '_diffrn_ambient_pressure' => ...,
+#                '_diffrn.ambient_pressure' => ...,
+#                '_diffrn.ambient_pressure_esd' => ...,
+#                '_diffrn_ambient_pressure_gPa' => ...,
+#                '_diffrn_ambient_pressure_gt' => ...,
+#                '_diffrn.ambient_pressure_gt' => ...,
+#                '_diffrn_ambient_pressure_lt' => ...,
+#                '_diffrn.ambient_pressure_lt' => ...,
+#                '_pd_prep_pressure' => ...,
+#           },
+#         # Temperature under which the crystal was measured
+#           'temperature' => {
+#                '_cell_measurement_temperature' => 293.2
+#                '_cell_measurement_temperature_C' => ...,
+#                '_diffrn_ambient_temperature' => ...,
+#                '_diffrn_ambient_temperature_C' => ...,
+#                '_diffrn_ambient_temperature_gt' => ...,
+#                '_diffrn_ambient_temperature_lt' => ...,
+#                '_pd_prep_temperature' => ...,
+#           },
+#         # Compound source
+#           'source' => {
+#                '_chemical_compound_source' => 'The moon',
+#           },
+#         # COD IDs of COD entries that are explicitly marked
+#         # as enantiomers of this structures
+#           'related_enantiomer_entries' => [
+#               '1000001',
+#               '1000002',
+#           ],
+#         # COD ID of the COD entry that is explicitly
+#         # marked as the optimal solution of this structure
+#           'related_optimal' => '1000003',
+#         # 'yes'/'no' value denoting if the structure is suboptimal
+#           'suboptimal' => 'yes',
+#       }
+##
 sub cif_fill_data
 {
-    my ( $dataset, $file, $index ) = @_;
+    my ( $dataset, $file, $index, $options ) = @_;
+
+    $options = {} unless $options;
+    my $use_attached_hydrogens =
+                        defined $options->{'use_attached_hydrogens'} ?
+                                $options->{'use_attached_hydrogens'} : 0;
 
     my %structure;
 
-    my $values = $dataset->{values};
-    my $sigmas = $dataset->{precisions};
-    my $id = $dataset->{name};
+    my $values = $dataset->{'values'};
+    my $sigmas = $dataset->{'precisions'};
+    my $id = $dataset->{'name'};
 
     return if !defined $id;
-    $structure{id} = $id;
-    $structure{filename} = basename( $file );
-    $structure{index} = $index;
+    $structure{'id'} = $id;
+    $structure{'filename'} = basename( $file );
+    $structure{'index'} = $index;
 
-    if( defined $values->{_chemical_formula_sum} ) {
-        my $formula = $values->{_chemical_formula_sum}[0];
+    if( defined $values->{'_chemical_formula_sum'} ) {
+        my $formula = $values->{'_chemical_formula_sum'}[0];
 
         if( $formula ne '?' ) {
 
@@ -264,37 +393,38 @@ sub cif_fill_data
                 $formula = join ' ', sort {$a cmp $b} split( ' ', $formula );
             }
         }
-        $structure{chemical_formula_sum} = $formula;
+        $structure{'chemical_formula_sum'} = $formula;
     }
 
     my $calc_formula;
     eval {
-        $calc_formula = cif_cell_contents( $dataset, undef );
+        $calc_formula = cif_cell_contents( $dataset,
+                                           undef,
+                                           $use_attached_hydrogens );
     };
     if ($@) {
-        # ERRORs that originated within the function are downgraded to warnings
+        # ERRORs that originate within the function are downgraded to warnings
         my $error = $@;
         $error =~ s/[A-Z]+, //;
         chomp $error;
         warn "WARNING, summary formula could not be calculated -- $error\n";
     };
-    $structure{calc_formula} = $calc_formula
-        if defined $calc_formula;
-
+    $structure{'calc_formula'} = $calc_formula if defined $calc_formula;
 
     my $cell_formula;
     eval {
-        $cell_formula = cif_cell_contents( $dataset, 1 );
+        $cell_formula = cif_cell_contents( $dataset,
+                                           1,
+                                           $use_attached_hydrogens );
     };
     if ($@) {
-        # ERRORs that originated within the function are downgraded to warnings
+        # ERRORs that originate within the function are downgraded to warnings
         my $error = $@;
         $error =~ s/[A-Z]+, //;
         chomp $error;
         warn "WARNING, unit cell summary formula could not be calculated -- $error\n";
     };
-    $structure{cell_formula} = $cell_formula
-        if defined $cell_formula;
+    $structure{'cell_formula'} = $cell_formula if defined $cell_formula;
 
     for my $key ( qw( _cell_length_a
                       _cell_length_b
@@ -328,8 +458,8 @@ sub cif_fill_data
         ],
         'history' => [
             qw(
-                 _exptl_crystal_pressure_history
-                 _exptl_crystal_thermal_history
+                _exptl_crystal_pressure_history
+                _exptl_crystal_thermal_history
             )
         ],
         'pressure' => [
@@ -360,17 +490,6 @@ sub cif_fill_data
                 _pd_prep_temperature
             )
         ],
-        # FIXME: the 'enantiomer' field should be removed prior to a major
-        # release since it was replaced with the 'related_enantiomer_entries'
-        # field. The new field takes into consideration that there might be
-        # more than one related enantiomer entry.
-        'enantiomer' => [
-            qw(
-                _cod_related_enantiomer_entry.code
-                _cod_related_enantiomer_entry_code
-                _cod_enantiomer_of
-            )
-        ],
         'source' => [
             qw(
                 _chemical_compound_source
@@ -502,11 +621,14 @@ sub have_equiv_lattices
     my ($entry1, $entry2, $options) = @_;
 
     my $use_su          = defined $options->{'use_su'} ?
-                                  $options->{'use_su'} : 1;
+                                  $options->{'use_su'} :
+                                  $DEFAULT_OPTIONS{'use_su'};
     my $max_length_diff = defined $options->{'max_cell_length_diff'} ?
-                                  $options->{'max_cell_length_diff'} : 0.5;
+                                  $options->{'max_cell_length_diff'} :
+                                  $DEFAULT_OPTIONS{'max_cell_length_diff'};
     my $max_angle_diff  = defined $options->{'max_cell_angle_diff'} ?
-                                  $options->{'max_cell_angle_diff'} : 1.2;
+                                  $options->{'max_cell_angle_diff'} :
+                                  $DEFAULT_OPTIONS{'max_cell_angle_diff'};
 
     my @cell1  = get_cell( $entry1->{'cell'} );
     my @cell2  = get_cell( $entry2->{'cell'} );
@@ -604,7 +726,7 @@ sub are_equiv_meas
     my $max_diff = defined $options->{'max_diff'} ?
                            $options->{'max_diff'} : 0;
 
-    if( !(defined $su_1 || defined $su_2) ) {
+    if( !defined $su_1 && !defined $su_2 ) {
         $use_su = 0;
     } else {
         $su_1 = 0 if !defined $su_1;
@@ -850,9 +972,9 @@ sub entries_are_the_same
 
     $user_options = {} if !defined $user_options;
     my %options = %{$user_options};
-    foreach my $key (keys %default_options) {
+    foreach my $key (keys %DEFAULT_OPTIONS) {
         if ( !defined $options{$key} ) {
-            $options{$key} = $default_options{$key};
+            $options{$key} = $DEFAULT_OPTIONS{$key};
         }
     };
 
@@ -937,23 +1059,6 @@ sub is_related_enantiomer_entry
     my ($main_entry, $related_entry) = @_;
 
     my @enantiomer_entries;
-
-    # FIXME: the following code block should be removed once
-    # the 'enantiomer' entry field is properly deprecated and removed 
-    # BEGIN
-    if (defined $main_entry->{'enantiomer'}) {
-        my @enantiomer_tags = qw(
-            _cod_related_enantiomer_entry.code
-            _cod_related_enantiomer_entry_code
-            _cod_enantiomer_of
-        );
-        for my $tag (@enantiomer_tags) {
-            next if !defined $main_entry->{'enantiomer'}{$tag};
-            push @enantiomer_entries, $main_entry->{'enantiomer'}{$tag};
-        }
-    }
-    # END
-
     if (exists $main_entry->{'related_enantiomer_entries'}) {
         push @enantiomer_entries, @{$main_entry->{'related_enantiomer_entries'}};
     }
@@ -1019,9 +1124,9 @@ sub prepare_database_query
 
     $user_options = {} if !defined $user_options;
     my %options = %{$user_options};
-    foreach my $key (keys %default_options) {
+    foreach my $key (keys %DEFAULT_OPTIONS) {
         if ( !defined $options{$key} ) {
-            $options{$key} = $default_options{$key};
+            $options{$key} = $DEFAULT_OPTIONS{$key};
         }
     };
     my $cod_series_prefix = defined $options{'cod_series_prefix'} ?
