@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
 #$Author: antanas $
-#$Date: 2019-11-18 11:08:55 +0200 (Mon, 18 Nov 2019) $ 
-#$Revision: 7429 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/CIF/Data/CellContents.pm $
+#$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $ 
+#$Revision: 8230 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/CIF/Data/CellContents.pm $
 #------------------------------------------------------------------------------
 #*
 #  Calculate unit cell contents from the atomic coordinates and
@@ -75,7 +75,7 @@ sub cif_cell_contents( $$@ )
     ## serialiseRef( \@sym_operators );
 
     my $sym_atoms =
-        symop_generate_atoms( \@sym_operators, $atoms, $ortho_matrix,
+        symop_generate_atoms( \@sym_operators, $atoms,
                               { use_special_position_disorder => 1 } );
 
     ## serialiseRef( $sym_atoms );
@@ -133,7 +133,7 @@ sub atomic_composition($$$@)
 {
     my ( $sym_atoms, $Z, $gp_multiplicity, $use_attached_hydrogens,
          $assume_full_occupancies ) = @_;
-    $use_attached_hydrogens = 0 unless defined $use_attached_hydrogens;
+    $use_attached_hydrogens  = 1 unless defined $use_attached_hydrogens;
     $assume_full_occupancies = 0 unless defined $assume_full_occupancies;
     my %composition;
 
