@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
-#$Author: andrius $
-#$Date: 2017-09-28 16:59:34 +0300 (Thu, 28 Sep 2017) $ 
-#$Revision: 5556 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/CIF/Data/Classifier.pm $
+#$Author: antanas $
+#$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $ 
+#$Revision: 8230 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/CIF/Data/Classifier.pm $
 #------------------------------------------------------------------------------
 #*
 #  Classify a CIF structure -- find out if it is organic compound,
@@ -106,7 +106,7 @@ sub cif_has_C_bonds( $$$$ )
     ## use COD::Serialise qw( serialiseRef );
     ## serialiseRef( $g );
 
-    my $sym_atoms = symop_generate_atoms( \@symop_matrices, $atoms, $f2o );
+    my $sym_atoms = symop_generate_atoms( \@symop_matrices, $atoms );
 
     ## use COD::Serialise qw( serialiseRef );
     ## serialiseRef( $sym_atoms );
