@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
 #$Author: antanas $
-#$Date: 2019-11-15 17:58:23 +0200 (Fri, 15 Nov 2019) $
-#$Revision: 7411 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/CIF/Data/SymmetryGenerator.pm $
+#$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $
+#$Revision: 8230 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/CIF/Data/SymmetryGenerator.pm $
 #------------------------------------------------------------------------------
 #*
 #  Calculate unit cell contents from the atomic coordinates and
@@ -58,7 +58,7 @@ sub apply_shifts($);
 sub atoms_coincide($$$);
 sub chemical_formula_sum($@);
 sub symop_apply($$@);
-sub symop_generate_atoms($$$@);
+sub symop_generate_atoms($$@);
 sub symop_register_applied_symop($$@);
 sub symops_apply_modulo1($$@);
 sub test_bond($$$$$);
@@ -116,17 +116,12 @@ sub atoms_coincide($$$)
 #       atoms
 #                       Reference to an array of atoms, as extracted
 #                       via COD::CIF::Data::AtomList.
-#       f2o
-#                       Reference to an array with fractional-to-ortho
-#                       matrix. As currently such matrix is taken from
-#                       atom description, parameter 'f2o' is not used
-#                       and will be removed in next major version.
 #       options
 #                       Reference to a hash of options.
 
-sub symop_generate_atoms($$$@)
+sub symop_generate_atoms($$@)
 {
-    my ( $sym_operators, $atoms, $f2o, $options ) = @_;
+    my ( $sym_operators, $atoms, $options ) = @_;
 
     $options = {} unless $options;
 
