@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
-#$Author: andrius $
-#$Date: 2017-09-15 12:47:32 +0300 (Fri, 15 Sep 2017) $ 
-#$Revision: 5515 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.9/src/lib/perl5/COD/CIF/Parser.pm $
+#$Author: antanas $
+#$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $ 
+#$Revision: 8230 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/src/lib/perl5/COD/CIF/Parser.pm $
 #------------------------------------------------------------------------------
 #*
 #  Subroutines that provide a functional interface to COD::CIF::Parser::Yapp
@@ -14,7 +14,7 @@ package COD::CIF::Parser;
 use strict;
 use warnings;
 use COD::CIF::Parser::Yapp;
-use COD::UserMessage qw( sprint_message warning error );
+use COD::UserMessage qw( warning error );
 use COD::CIF::JSON;
 
 require Exporter;
@@ -47,16 +47,24 @@ sub parse_cif
     if ( !defined $options ) {
         $options = {};
     } elsif( ref $options ne "HASH" ) {
-        error( $0, $filename, undef, 'options for the CIF parser should be '
-             . 'provided via a hash reference', undef );
+        error( {
+            'program'  => $0,
+            'filename' => $filename,
+            'message'  =>
+                'options for the CIF parser should be provided ' .
+                'via a hash reference'
+        } );
         exit 1;
     }
 
     my $unrecognised;
     ($options, $unrecognised) = check_options($options, $default_options);
     foreach ( @$unrecognised ) {
-        warning($0, $filename, undef, "option '$_' is "
-              . "not supported by the CIF parser", undef);
+        warning( {
+            'program'  => $0,
+            'filename' => $filename,
+            'message'  => "option '$_' is not supported by the CIF parser"
+        } );
     }
 
     my $parser;
@@ -68,17 +76,25 @@ sub parse_cif
     } elsif ( $options->{parser} eq 'json' ) {
         $parser = new COD::CIF::JSON;
     } else {
-        error( $0, $filename, undef, "parser type '" . $options->{parser}
-             . "' is not recognised", "please select either 'c' or 'perl' "
-             . 'CIF parsers' );
+        error( {
+            'program'  => $0,
+            'filename' => $filename,
+            'message'  =>
+                "parser type '" . $options->{parser} . "' is not recognised " .
+                "-- please select either 'c' or 'perl' CIF parsers"
+        } );
         exit 1;
     }
 
     if ($options->{parser} eq 'c') {
         if ( defined $options->{reporter} ) {
-            warning( $0, $filename, undef, "option 'reporter' is only "
-                   . 'supported by the perl CIF parser',
-                     'option will be ignored' );
+            warning( {
+                'program'  => $0,
+                'filename' => $filename,
+                'message'  =>
+                    "option 'reporter' is only supported by the perl " .
+                    'CIF parser -- option will be ignored'
+            } );
             delete $options->{reporter};
         }
     }
