@@ -7,9 +7,9 @@ INPUT_MODULES='src/lib/perl5/COD/UserMessage.pm'
 perl <<'END_SCRIPT'
 #------------------------------------------------------------------------------
 #$Author: antanas $
-#$Date: 2018-08-06 17:25:17 +0300 (Mon, 06 Aug 2018) $ 
-#$Revision: 6377 $
-#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v2.12/tests/shtests/sprint_message_001.sh $
+#$Date: 2020-07-20 16:43:00 +0300 (Mon, 20 Jul 2020) $ 
+#$Revision: 8230 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.0.1/tests/shtests/sprint_message_001.sh $
 #------------------------------------------------------------------------------
 #**
 #* Unit test for the COD::UserMessage::sprint_message() subroutine.
@@ -26,17 +26,16 @@ my @test_symbols = ( ' ', "\n", "\t", '(', ')', '{', '}', '[', ']' );
 
 my $message;
 for my $test_symbol ( @test_symbols ) {
-    $message = sprint_message(
-        "program${test_symbol}",
-        "filename${test_symbol}",
-        "data_block${test_symbol}",
-        'WARNING',
-        'message',
-        undef,
-        1,
-        2,
-        'an error line'
-    );
+    $message = sprint_message( {
+        'program'      => "program${test_symbol}",
+        'filename'     => "filename${test_symbol}",
+        'add_pos'      => "data_block${test_symbol}",
+        'err_level'    => 'WARNING',
+        'message'      => 'message',
+        'line_no'      => 1,
+        'column_no'    => 2,
+        'line_content' => 'an error line'
+    } );
     print "# Contains the '$test_symbol' symbol:\n";
     print $message;
 }
